# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

from abc import abstractmethod
from os import PathLike
from pathlib import Path
from typing import IO, Any, AnyStr, Dict, Optional, Union, cast

from azure.ai.ml._restclient.v2022_10_01_preview.models import ComputeResource
from azure.ai.ml._schema.compute.compute import ComputeSchema
from azure.ai.ml._utils.utils import dump_yaml_to_file
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, PARAMS_OVERRIDE_KEY, CommonYamlFields
from azure.ai.ml.constants._compute import ComputeType
from azure.ai.ml.entities._mixins import RestTranslatableMixin
from azure.ai.ml.entities._resource import Resource
from azure.ai.ml.entities._util import find_type_in_override
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException


class Compute(Resource, RestTranslatableMixin):
    """Base class for compute resources.

    This class should not be instantiated directly. Instead, use one of its subclasses.

    :param type: The compute type. Accepted values are "amlcompute", "computeinstance",
        "virtualmachine", "kubernetes", and "synapsespark".
    :type type: str
    :param name: Name of the compute resource.
    :type name: str
    :param location: The resource location. Defaults to workspace location.
    :type location: Optional[str]
    :param description: Description of the resource. Defaults to None.
    :type description: Optional[str]
    :param resource_id: ARM resource id of the underlying compute. Defaults to None.
    :type resource_id: Optional[str]
    :param tags: A set of tags. Contains resource tags defined as key/value pairs.
    :type tags: Optional[dict[str, str]]
    """

    def __init__(
        self,
        name: str,
        location: Optional[str] = None,
        description: Optional[str] = None,
        resource_id: Optional[str] = None,
        tags: Optional[Dict] = None,
        **kwargs: Any,
    ) -> None:
        self._type: Optional[str] = kwargs.pop("type", None)
        if self._type:
            self._type = self._type.lower()

        self._created_on: Optional[str] = kwargs.pop("created_on", None)
        self._provisioning_state: Optional[str] = kwargs.pop("provisioning_state", None)
        self._provisioning_errors: Optional[str] = kwargs.pop("provisioning_errors", None)

        super().__init__(name=name, description=description, **kwargs)
        self.resource_id = resource_id
        self.location = location
        self.tags = tags

    @property
    def type(self) -> Optional[str]:
        """The compute type.

        :return: The compute type.
        :rtype: Optional[str]
        """
        return self._type

    @property
    def created_on(self) -> Optional[str]:
        """The compute resource creation timestamp.

        :return: The compute resource creation timestamp.
        :rtype: Optional[str]
        """
        return self._created_on

    @property
    def provisioning_state(self) -> Optional[str]:
        """The compute resource's provisioning state.

        :return: The compute resource's provisioning state.
        :rtype: Optional[str]
        """
        return self._provisioning_state

    @property
    def provisioning_errors(self) -> Optional[str]:
        """The compute resource provisioning errors.

        :return: The compute resource provisioning errors.
        :rtype: Optional[str]
        """
        return self._provisioning_errors

    def _to_rest_object(self) -> ComputeResource:
        pass

    @classmethod
    def _from_rest_object(cls, obj: ComputeResource) -> "Compute":
        from azure.ai.ml.entities import (
            AmlCompute,
            ComputeInstance,
            KubernetesCompute,
            SynapseSparkCompute,
            UnsupportedCompute,
            VirtualMachineCompute,
        )

        mapping = {
            ComputeType.AMLCOMPUTE.lower(): AmlCompute,
            ComputeType.COMPUTEINSTANCE.lower(): ComputeInstance,
            ComputeType.VIRTUALMACHINE.lower(): VirtualMachineCompute,
            ComputeType.KUBERNETES.lower(): KubernetesCompute,
            ComputeType.SYNAPSESPARK.lower(): SynapseSparkCompute,
        }
        compute_type = obj.properties.compute_type.lower() if obj.properties.compute_type else None

        class_type = cast(
            Optional[Union[AmlCompute, ComputeInstance, VirtualMachineCompute, KubernetesCompute, SynapseSparkCompute]],
            mapping.get(compute_type, None),  # type: ignore
        )
        if class_type:
            return class_type._load_from_rest(obj)
        _unsupported_from_rest: Compute = UnsupportedCompute._load_from_rest(obj)
        return _unsupported_from_rest

    @classmethod
    @abstractmethod
    def _load_from_rest(cls, rest_obj: ComputeResource) -> "Compute":
        pass

    def _set_full_subnet_name(self, subscription_id: str, rg: str) -> None:
        pass

    def dump(self, dest: Union[str, PathLike, IO[AnyStr]], **kwargs: Any) -> None:
        """Dump the compute content into a file in yaml format.

        :param dest: The destination to receive this compute's content.
            Must be either a path to a local file, or an already-open file stream.
            If dest is a file path, a new file will be created,
            and an exception is raised if the file exists.
            If dest is an open file, the file will be written to directly,
            and an exception will be raised if the file is not writable.'.
        :type dest: Union[PathLike, str, IO[AnyStr]]
        """
        path = kwargs.pop("path", None)
        yaml_serialized = self._to_dict()
        dump_yaml_to_file(dest, yaml_serialized, default_flow_style=False, path=path, **kwargs)

    def _to_dict(self) -> Dict:
        res: dict = ComputeSchema(context={BASE_PATH_CONTEXT_KEY: "./"}).dump(self)
        return res

    @classmethod
    def _load(
        cls,
        data: Optional[Dict] = None,
        yaml_path: Optional[Union[PathLike, str]] = None,
        params_override: Optional[list] = None,
        **kwargs: Any,
    ) -> "Compute":
        data = data or {}
        params_override = params_override or []
        context = {
            BASE_PATH_CONTEXT_KEY: Path(yaml_path).parent if yaml_path else Path("./"),
            PARAMS_OVERRIDE_KEY: params_override,
        }
        from azure.ai.ml.entities import (
            AmlCompute,
            ComputeInstance,
            KubernetesCompute,
            SynapseSparkCompute,
            VirtualMachineCompute,
        )

        type_in_override = find_type_in_override(params_override) if params_override else None
        compute_type = type_in_override or data.get(CommonYamlFields.TYPE, None)  # override takes the priority.
        if compute_type:
            if compute_type.lower() == ComputeType.VIRTUALMACHINE:
                _vm_load_from_dict: Compute = VirtualMachineCompute._load_from_dict(data, context, **kwargs)
                return _vm_load_from_dict
            if compute_type.lower() == ComputeType.AMLCOMPUTE:
                _aml_load_from_dict: Compute = AmlCompute._load_from_dict(data, context, **kwargs)
                return _aml_load_from_dict
            if compute_type.lower() == ComputeType.COMPUTEINSTANCE:
                _compute_instance_load_from_dict: Compute = ComputeInstance._load_from_dict(data, context, **kwargs)
                return _compute_instance_load_from_dict
            if compute_type.lower() == ComputeType.KUBERNETES:
                _kub_load_from_dict: Compute = KubernetesCompute._load_from_dict(data, context, **kwargs)
                return _kub_load_from_dict
            if compute_type.lower() == ComputeType.SYNAPSESPARK:
                _synapse_spark_load_from_dict: Compute = SynapseSparkCompute._load_from_dict(data, context, **kwargs)
                return _synapse_spark_load_from_dict
        msg = f"Unknown compute type: {compute_type}"
        raise ValidationException(
            message=msg,
            target=ErrorTarget.COMPUTE,
            no_personal_data_message=msg,
            error_category=ErrorCategory.USER_ERROR,
        )

    @classmethod
    @abstractmethod
    def _load_from_dict(cls, data: Dict, context: Dict, **kwargs: Any) -> "Compute":
        pass


class NetworkSettings:
    """Network settings for a compute resource. If the workspace and VNet are in different resource groups,
    please provide the full URI for subnet and leave vnet_name as None.

    :param vnet_name: The virtual network name.
    :type vnet_name: Optional[str]
    :param subnet: The subnet name.
    :type subnet: Optional[str]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_compute.py
            :start-after: [START network_settings]
            :end-before: [END network_settings]
            :language: python
            :dedent: 8
            :caption: Configuring NetworkSettings for an AmlCompute object.
    """

    def __init__(
        self,
        *,
        vnet_name: Optional[str] = None,
        subnet: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        self.vnet_name = vnet_name
        self.subnet = subnet
        self._public_ip_address: str = kwargs.pop("public_ip_address", None)
        self._private_ip_address: str = kwargs.pop("private_ip_address", None)

    @property
    def public_ip_address(self) -> str:
        """Public IP address of the compute instance.

        :return: Public IP address.
        :rtype: str
        """
        return self._public_ip_address

    @property
    def private_ip_address(self) -> str:
        """Private IP address of the compute instance.

        :return: Private IP address.
        :rtype: str
        """
        return self._private_ip_address



import argparse

import mlflow.sklearn
import numpy as np
from sklearn.svm import SVC


def parse_args():
    parser = argparse.ArgumentParser(description="SVM example")
    parser.add_argument(
        "-c",
        type=float,
        default=1.0,
        help="SVM Cost Parameter",
    )
    return parser.parse_args()


def main():
    args = parse_args()
    mlflow.sklearn.autolog()
    X = np.array([[-1, -1], [-3, -1], [0, 1], [1, 1], [5, 4]])
    y = np.array([0, 0, 0, 1, 1])

    # junk test data so we can log a metric. accuracy should be bad due to not many training points
    test_data = np.random.rand(10, 2)
    test_labels = np.random.randint(2, size=10)
    clf = SVC(C=args.c)
    with mlflow.start_run():
        clf.fit(X, y)
        pred = clf.predict(test_data)
        accuracy = np.sum(test_labels == pred) / 10
        mlflow.log_metric("accuracy", accuracy)
        mlflow.log_param("a_param", 1)
        mlflow.log_param("another_param", 2)


if __name__ == "__main__":
    main()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

__path__ = __import__("pkgutil").extend_path(__path__, __name__)

from azure.ai.ml.entities._job.distillation.endpoint_request_settings import EndpointRequestSettings
from azure.ai.ml.entities._job.distillation.prompt_settings import PromptSettings
from azure.ai.ml.entities._job.distillation.teacher_model_settings import TeacherModelSettings

from ._distillation import distillation

__all__ = ["distillation", "EndpointRequestSettings", "PromptSettings", "TeacherModelSettings"]



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
from typing import IO, Any, Callable, Dict, Optional, TypeVar, Union, cast, overload

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._managed_network_provisions_operations import build_provision_managed_network_request

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class ManagedNetworkProvisionsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.machinelearningservices.aio.AzureMachineLearningServices`'s
        :attr:`managed_network_provisions` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    async def _provision_managed_network_initial(
        self,
        resource_group_name: str,
        workspace_name: str,
        body: Optional[Union[_models.ManagedNetworkProvisionOptions, IO]] = None,
        **kwargs: Any
    ) -> Optional[_models.ManagedNetworkProvisionStatus]:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Optional[_models.ManagedNetworkProvisionStatus]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            if body is not None:
                _json = self._serialize.body(body, "ManagedNetworkProvisionOptions")
            else:
                _json = None

        request = build_provision_managed_network_request(
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self._provision_managed_network_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = None
        response_headers = {}
        if response.status_code == 200:
            deserialized = self._deserialize("ManagedNetworkProvisionStatus", pipeline_response)

        if response.status_code == 202:
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))
            response_headers["Retry-After"] = self._deserialize("int", response.headers.get("Retry-After"))

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    _provision_managed_network_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/provisionManagedNetwork"
    }

    @overload
    async def begin_provision_managed_network(
        self,
        resource_group_name: str,
        workspace_name: str,
        body: Optional[_models.ManagedNetworkProvisionOptions] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.ManagedNetworkProvisionStatus]:
        """Provisions the managed network of a machine learning workspace.

        Provisions the managed network of a machine learning workspace.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param body: Managed Network Provisioning Options for a machine learning workspace. Default
         value is None.
        :type body: ~azure.mgmt.machinelearningservices.models.ManagedNetworkProvisionOptions
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either ManagedNetworkProvisionStatus or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.machinelearningservices.models.ManagedNetworkProvisionStatus]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_provision_managed_network(
        self,
        resource_group_name: str,
        workspace_name: str,
        body: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.ManagedNetworkProvisionStatus]:
        """Provisions the managed network of a machine learning workspace.

        Provisions the managed network of a machine learning workspace.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param body: Managed Network Provisioning Options for a machine learning workspace. Default
         value is None.
        :type body: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either ManagedNetworkProvisionStatus or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.machinelearningservices.models.ManagedNetworkProvisionStatus]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_provision_managed_network(
        self,
        resource_group_name: str,
        workspace_name: str,
        body: Optional[Union[_models.ManagedNetworkProvisionOptions, IO]] = None,
        **kwargs: Any
    ) -> AsyncLROPoller[_models.ManagedNetworkProvisionStatus]:
        """Provisions the managed network of a machine learning workspace.

        Provisions the managed network of a machine learning workspace.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace. Required.
        :type workspace_name: str
        :param body: Managed Network Provisioning Options for a machine learning workspace. Is either a
         ManagedNetworkProvisionOptions type or a IO type. Default value is None.
        :type body: ~azure.mgmt.machinelearningservices.models.ManagedNetworkProvisionOptions or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either ManagedNetworkProvisionStatus or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.machinelearningservices.models.ManagedNetworkProvisionStatus]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ManagedNetworkProvisionStatus] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._provision_managed_network_initial(
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                body=body,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("ManagedNetworkProvisionStatus", pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod, AsyncARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_provision_managed_network.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/provisionManagedNetwork"
    }



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import TYPE_CHECKING

from msrest import Serializer

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._vendor import _convert_request, _format_url_section

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Optional, TypeVar
    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False
# fmt: off

def build_batch_post_by_experiment_name_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    experiment_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experiments/{experimentName}/batch/events")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "experimentName": _SERIALIZER.url("experiment_name", experiment_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=_url,
        headers=_header_parameters,
        **kwargs
    )


def build_batch_post_by_experiment_id_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    experiment_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experimentids/{experimentId}/batch/events")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "experimentId": _SERIALIZER.url("experiment_id", experiment_id, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=_url,
        headers=_header_parameters,
        **kwargs
    )


def build_batch_post_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/batch/events")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=_url,
        headers=_header_parameters,
        **kwargs
    )


def build_post_by_experiment_name_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    run_id,  # type: str
    experiment_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experiments/{experimentName}/runs/{runId}/events")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "runId": _SERIALIZER.url("run_id", run_id, 'str'),
        "experimentName": _SERIALIZER.url("experiment_name", experiment_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=_url,
        headers=_header_parameters,
        **kwargs
    )


def build_post_by_experiment_id_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    run_id,  # type: str
    experiment_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experimentids/{experimentId}/runs/{runId}/events")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "runId": _SERIALIZER.url("run_id", run_id, 'str'),
        "experimentId": _SERIALIZER.url("experiment_id", experiment_id, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=_url,
        headers=_header_parameters,
        **kwargs
    )


def build_post_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    run_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/runs/{runId}/events")  # pylint: disable=line-too-long
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "runId": _SERIALIZER.url("run_id", run_id, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        _header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=_url,
        headers=_header_parameters,
        **kwargs
    )

# fmt: on
class EventsOperations(object):
    """EventsOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.machinelearningservices.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def batch_post_by_experiment_name(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        experiment_name,  # type: str
        body=None,  # type: Optional["_models.BatchEventCommand"]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.BatchEventCommandResult"
        """batch_post_by_experiment_name.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param experiment_name:
        :type experiment_name: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.BatchEventCommand
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: BatchEventCommandResult, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.BatchEventCommandResult
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.BatchEventCommandResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'BatchEventCommand')
        else:
            _json = None

        request = build_batch_post_by_experiment_name_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            experiment_name=experiment_name,
            content_type=content_type,
            json=_json,
            template_url=self.batch_post_by_experiment_name.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('BatchEventCommandResult', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    batch_post_by_experiment_name.metadata = {'url': "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experiments/{experimentName}/batch/events"}  # type: ignore


    @distributed_trace
    def batch_post_by_experiment_id(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        experiment_id,  # type: str
        body=None,  # type: Optional["_models.BatchEventCommand"]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.BatchEventCommandResult"
        """batch_post_by_experiment_id.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param experiment_id:
        :type experiment_id: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.BatchEventCommand
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: BatchEventCommandResult, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.BatchEventCommandResult
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.BatchEventCommandResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'BatchEventCommand')
        else:
            _json = None

        request = build_batch_post_by_experiment_id_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            experiment_id=experiment_id,
            content_type=content_type,
            json=_json,
            template_url=self.batch_post_by_experiment_id.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('BatchEventCommandResult', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    batch_post_by_experiment_id.metadata = {'url': "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experimentids/{experimentId}/batch/events"}  # type: ignore


    @distributed_trace
    def batch_post(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        body=None,  # type: Optional["_models.BatchEventCommand"]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.BatchEventCommandResult"
        """batch_post.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.BatchEventCommand
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: BatchEventCommandResult, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.BatchEventCommandResult
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.BatchEventCommandResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'BatchEventCommand')
        else:
            _json = None

        request = build_batch_post_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            content_type=content_type,
            json=_json,
            template_url=self.batch_post.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('BatchEventCommandResult', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    batch_post.metadata = {'url': "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/batch/events"}  # type: ignore


    @distributed_trace
    def post_by_experiment_name(  # pylint: disable=inconsistent-return-statements
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        run_id,  # type: str
        experiment_name,  # type: str
        body=None,  # type: Optional["_models.BaseEvent"]
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """post_by_experiment_name.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param run_id:
        :type run_id: str
        :param experiment_name:
        :type experiment_name: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.BaseEvent
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'BaseEvent')
        else:
            _json = None

        request = build_post_by_experiment_name_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            run_id=run_id,
            experiment_name=experiment_name,
            content_type=content_type,
            json=_json,
            template_url=self.post_by_experiment_name.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in []:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    post_by_experiment_name.metadata = {'url': "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experiments/{experimentName}/runs/{runId}/events"}  # type: ignore


    @distributed_trace
    def post_by_experiment_id(  # pylint: disable=inconsistent-return-statements
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        run_id,  # type: str
        experiment_id,  # type: str
        body=None,  # type: Optional["_models.BaseEvent"]
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """post_by_experiment_id.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param run_id:
        :type run_id: str
        :param experiment_id:
        :type experiment_id: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.BaseEvent
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'BaseEvent')
        else:
            _json = None

        request = build_post_by_experiment_id_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            run_id=run_id,
            experiment_id=experiment_id,
            content_type=content_type,
            json=_json,
            template_url=self.post_by_experiment_id.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in []:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    post_by_experiment_id.metadata = {'url': "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/experimentids/{experimentId}/runs/{runId}/events"}  # type: ignore


    @distributed_trace
    def post(  # pylint: disable=inconsistent-return-statements
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        run_id,  # type: str
        body=None,  # type: Optional["_models.BaseEvent"]
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """post.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param run_id:
        :type run_id: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.BaseEvent
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'BaseEvent')
        else:
            _json = None

        request = build_post_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            run_id=run_id,
            content_type=content_type,
            json=_json,
            template_url=self.post.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in []:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    post.metadata = {'url': "/history/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/runs/{runId}/events"}  # type: ignore




# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


from functools import wraps
from typing import Any, Callable, Optional

from typing_extensions import Concatenate, ParamSpec, Self

from azure.core.configuration import Configuration
from azure.core.pipeline import Pipeline
from azure.core.pipeline.policies import (
    CustomHookPolicy,
    HeadersPolicy,
    HttpLoggingPolicy,
    NetworkTraceLoggingPolicy,
    ProxyPolicy,
    RedirectPolicy,
    RetryPolicy,
    UserAgentPolicy,
)
from azure.core.pipeline.transport import (  # pylint: disable=non-abstract-transport-import,no-name-in-module
    HttpTransport,
    RequestsTransport,
)
from azure.core.rest import HttpRequest, HttpResponse


def _request_function(f: Callable[["HttpPipeline"], None]):
    """A decorator that provides the implementation for the request-style convenience functions of the HttpPipeline
    class.

    :param Callable[[], None] f: A function whose name will be used as the http
                                 request method
    :return: An HTTP request function
    :rtype: Callable
    """

    # This is a hack to provide richer typing for the decorated function
    # The outer _ function allows us to pattern match on the parameter types
    # of HttpRequest and forward those types to the decorated function
    P = ParamSpec("P")

    def _(_: Callable[Concatenate[str, P], Any] = HttpRequest):
        @wraps(f)
        # pylint: disable-next=docstring-missing-param
        def decorated(self: "HttpPipeline", *args: P.args, **kwargs: P.kwargs) -> HttpResponse:
            """A function that sends an HTTP request and returns the response.

            Accepts the same parameters as azure.core.rest.HttpRequest, except for the method.
            All other kwargs are forwarded to azure.core.Pipeline.run

            :return: The request response
            :rtype: HttpResponse
            """
            request = HttpRequest(
                f.__name__.upper(),
                *args,
                params=kwargs.pop("params", None),
                headers=kwargs.pop("headers", None),
                json=kwargs.pop("json", None),
                content=kwargs.pop("content", None),
                data=kwargs.pop("data", None),
                files=kwargs.pop("files", None),
            )

            return self.run(request, **kwargs).http_response

        return decorated

    return _()


class HttpPipeline(Pipeline):
    """A *very* thin wrapper over azure.core.pipeline.Pipeline that facilitates sending miscellaneous http requests by
    adding:

    * A requests-style api for sending http requests
    * Facilities for populating policies for the client, include defaults,
     and re-using policies from an existing client.
    """

    def __init__(
        self,
        *,
        transport: Optional[HttpTransport] = None,
        config: Optional[Configuration] = None,
        user_agent_policy: Optional[UserAgentPolicy] = None,
        headers_policy: Optional[HeadersPolicy] = None,
        proxy_policy: Optional[ProxyPolicy] = None,
        logging_policy: Optional[NetworkTraceLoggingPolicy] = None,
        http_logging_policy: Optional[HttpLoggingPolicy] = None,
        retry_policy: Optional[RetryPolicy] = None,
        custom_hook_policy: Optional[CustomHookPolicy] = None,
        redirect_policy: Optional[RedirectPolicy] = None,
        **kwargs
    ):
        """

        :param HttpTransport transport: Http Transport used for requests, defaults to RequestsTransport
        :param Configuration config:
        :param UserAgentPolicy user_agent_policy:
        :param HeadersPolicy headers_policy:
        :param ProxyPolicy proxy_policy:
        :param NetworkTraceLoggingPolicy logging_policy:
        :param HttpLoggingPolicy http_logging_policy:
        :param RetryPolicy retry_policy:
        :param CustomHookPolicy custom_hook_policy:
        :param RedirectPolicy redirect_policy:
        """
        config = config or Configuration()
        config.headers_policy = headers_policy or config.headers_policy or HeadersPolicy(**kwargs)
        config.proxy_policy = proxy_policy or config.proxy_policy or ProxyPolicy(**kwargs)
        config.redirect_policy = redirect_policy or config.redirect_policy or RedirectPolicy(**kwargs)
        config.retry_policy = retry_policy or config.retry_policy or RetryPolicy(**kwargs)
        config.custom_hook_policy = custom_hook_policy or config.custom_hook_policy or CustomHookPolicy(**kwargs)
        config.logging_policy = logging_policy or config.logging_policy or NetworkTraceLoggingPolicy(**kwargs)
        config.http_logging_policy = http_logging_policy or config.http_logging_policy or HttpLoggingPolicy(**kwargs)
        config.user_agent_policy = user_agent_policy or config.user_agent_policy or UserAgentPolicy(**kwargs)
        config.polling_interval = kwargs.get("polling_interval", 30)

        super().__init__(
            # RequestsTransport normally should not be imported outside of azure.core, since transports
            # are meant to be user configurable.
            # RequestsTransport is only used in this file as the default transport when not user specified.
            transport=transport or RequestsTransport(**kwargs),
            policies=[
                config.headers_policy,
                config.user_agent_policy,
                config.proxy_policy,
                config.redirect_policy,
                config.retry_policy,
                config.authentication_policy,
                config.custom_hook_policy,
                config.logging_policy,
            ],
        )

        self._config = config

    def with_policies(self, **kwargs) -> Self:
        """A named constructor which facilitates creating a new pipeline using an existing one as a base.

           Accepts the same parameters as __init__

        :return: new Pipeline object with combined config of current object
            and specified overrides
        :rtype: Self
        """
        cls = self.__class__
        return cls(config=self._config, transport=kwargs.pop("transport", self._transport), **kwargs)

    @_request_function
    def delete(self) -> None:
        """Sends a DELETE request."""

    @_request_function
    def get(self) -> None:
        """Sends a GET request."""

    @_request_function
    def head(self) -> None:
        """Sends a HEAD request."""

    @_request_function
    def options(self) -> None:
        """Sends a OPTIONS request."""

    @_request_function
    def patch(self) -> None:
        """Sends a PATCH request."""

    @_request_function
    def post(self) -> None:
        """Sends a POST request."""

    @_request_function
    def put(self) -> None:
        """Sends a PUT request."""



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


import pytest
from devtools_testutils import AzureRecordedTestCase, is_live

from azure.ai.ml import MLClient
from test_common_functions import validate_job


@pytest.mark.finetuning_job_test
@pytest.mark.usefixtures("recorded_test")
@pytest.mark.skipif(condition=not is_live(), reason="This test requires a live endpoint")
class TestCustomModelFineTuningJob(AzureRecordedTestCase):

    @pytest.mark.e2etest
    def test_custom_model_finetuning_job(
        self,
        finetuning_job_with_name_updated,
        client: MLClient,
    ) -> None:

        # Trigger job
        created_job = client.jobs.create_or_update(finetuning_job_with_name_updated)
        returned_job = client.jobs.get(created_job.name)

        validate_job(
            input_job=finetuning_job_with_name_updated,
            created_job=created_job,
            returned_job=returned_job,
        )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


# ---------------------------------------------------------------------------------------------
# This package has been vendored from azure-identity package from the following commit
# https://github.com/Azure/azure-sdk-for-python/commit/0f302dc6c299df2ee637457c8f165c7bdb4ec2af
# ---------------------------------------------------------------------------------------------
from typing import Any

from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationException


# pylint: disable-next=docstring-missing-param,docstring-missing-return,docstring-missing-rtype
def _scopes_to_resource(*scopes: Any) -> Any:
    """Convert an AADv2 scope to an AADv1 resource."""

    if len(scopes) != 1:
        msg = "This credential requires exactly one scope per token request."
        raise ValidationException(
            message=msg,
            no_personal_data_message=msg,
            target=ErrorTarget.IDENTITY,
            error_category=ErrorCategory.USER_ERROR,
        )

    resource = scopes[0]
    if resource.endswith("/.default"):
        resource = resource[: -len("/.default")]

    return resource



import json
import logging
import os

import numpy
from sklearn.externals import joblib
from sklearn.linear_model import Ridge


def init():
    global model
    # AZUREML_MODEL_DIR is an environment variable created during deployment.
    # It is the path to the model folder (./azureml-models/$MODEL_NAME/$VERSION)
    # For multiple models, it points to the folder containing all deployed models (./azureml-models)
    model_path = os.path.join(os.getenv("AZUREML_MODEL_DIR"), "sklearn_regression_model.pkl")
    # deserialize the model file back into a sklearn model
    model = joblib.load(model_path)
    logging.info("Init complete")


# note you can pass in multiple rows for scoring
def run(raw_data):
    try:
        logging.info("Request received")
        data = json.loads(raw_data)["data"]
        data = numpy.array(data)
        result = model.predict(data)
        logging.info("Request processed")
        return result.tolist()
    except Exception as e:
        error = str(e)
        return error



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Optional, Union

from azure.ai.ml._restclient.v2023_04_01_preview.models import NlpSweepSettings as RestNlpSweepSettings
from azure.ai.ml._restclient.v2023_04_01_preview.models import SamplingAlgorithmType
from azure.ai.ml.entities._job.sweep.early_termination_policy import EarlyTerminationPolicy
from azure.ai.ml.entities._mixins import RestTranslatableMixin


# pylint: disable=protected-access
class NlpSweepSettings(RestTranslatableMixin):
    """Sweep settings for all AutoML NLP tasks.

    :param sampling_algorithm: Required. Specifies type of hyperparameter sampling algorithm.
        Possible values include: "Grid", "Random", and "Bayesian".
    :type sampling_algorithm: Union[str, ~azure.ai.ml.automl.SamplingAlgorithmType]
    :param early_termination: Early termination policy to end poorly performing training candidates,
        defaults to None.
    :type early_termination: Optional[~azure.mgmt.machinelearningservices.models.EarlyTerminationPolicy]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_automl_nlp.py
                :start-after: [START automl.nlp_sweep_settings]
                :end-before: [END automl.nlp_sweep_settings]
                :language: python
                :dedent: 8
                :caption: creating an nlp sweep settings
    """

    def __init__(
        self,
        *,
        sampling_algorithm: Union[str, SamplingAlgorithmType],
        early_termination: Optional[EarlyTerminationPolicy] = None,
    ):
        self.sampling_algorithm = sampling_algorithm
        self.early_termination = early_termination

    def _to_rest_object(self) -> RestNlpSweepSettings:
        return RestNlpSweepSettings(
            sampling_algorithm=self.sampling_algorithm,
            early_termination=self.early_termination._to_rest_object() if self.early_termination else None,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestNlpSweepSettings) -> "NlpSweepSettings":
        return cls(
            sampling_algorithm=obj.sampling_algorithm,
            early_termination=(
                EarlyTerminationPolicy._from_rest_object(obj.early_termination) if obj.early_termination else None
            ),
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, NlpSweepSettings):
            return NotImplemented

        return self.sampling_algorithm == other.sampling_algorithm and self.early_termination == other.early_termination

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import ActionResult
    from ._models_py3 import AssetId
    from ._models_py3 import BatchDataUriResponse
    from ._models_py3 import BatchGetResolvedURIs
    from ._models_py3 import ColumnDefinition
    from ._models_py3 import CreateUnregisteredInputData
    from ._models_py3 import CreateUnregisteredOutputData
    from ._models_py3 import DataCallRequest
    from ._models_py3 import DataContainer
    from ._models_py3 import DataContainerEntity
    from ._models_py3 import DataContainerMutable
    from ._models_py3 import DataField
    from ._models_py3 import DataUriV2Response
    from ._models_py3 import DataVersion
    from ._models_py3 import DataVersionEntity
    from ._models_py3 import DataVersionMutable
    from ._models_py3 import DataViewSetResult
    from ._models_py3 import Dataset
    from ._models_py3 import DatasetDefinition
    from ._models_py3 import DatasetDefinitionReference
    from ._models_py3 import DatasetPath
    from ._models_py3 import DatasetState
    from ._models_py3 import DatasetV2
    from ._models_py3 import EntityMetadata
    from ._models_py3 import ErrorAdditionalInfo
    from ._models_py3 import ErrorResponse
    from ._models_py3 import HistogramBin
    from ._models_py3 import HttpContent
    from ._models_py3 import HttpMethod
    from ._models_py3 import HttpRequestMessage
    from ._models_py3 import HttpResponseMessage
    from ._models_py3 import InnerErrorResponse
    from ._models_py3 import KeyValuePairStringIEnumerable1
    from ._models_py3 import LongRunningOperationResponse1LongRunningOperationResponseObject
    from ._models_py3 import Moments
    from ._models_py3 import PaginatedDataContainerEntityList
    from ._models_py3 import PaginatedDataVersionEntityList
    from ._models_py3 import PaginatedDatasetDefinitionList
    from ._models_py3 import PaginatedDatasetList
    from ._models_py3 import PaginatedDatasetV2List
    from ._models_py3 import PaginatedStringList
    from ._models_py3 import ProfileActionResult
    from ._models_py3 import ProfileResult
    from ._models_py3 import Quantiles
    from ._models_py3 import RegisterExistingData
    from ._models_py3 import RootError
    from ._models_py3 import STypeCount
    from ._models_py3 import SqlDataPath
    from ._models_py3 import StoredProcedureParameter
    from ._models_py3 import StringLengthCount
    from ._models_py3 import TypeCount
    from ._models_py3 import User
    from ._models_py3 import ValueCount
except (SyntaxError, ImportError):
    from ._models import ActionResult  # type: ignore
    from ._models import AssetId  # type: ignore
    from ._models import BatchDataUriResponse  # type: ignore
    from ._models import BatchGetResolvedURIs  # type: ignore
    from ._models import ColumnDefinition  # type: ignore
    from ._models import CreateUnregisteredInputData  # type: ignore
    from ._models import CreateUnregisteredOutputData  # type: ignore
    from ._models import DataCallRequest  # type: ignore
    from ._models import DataContainer  # type: ignore
    from ._models import DataContainerEntity  # type: ignore
    from ._models import DataContainerMutable  # type: ignore
    from ._models import DataField  # type: ignore
    from ._models import DataUriV2Response  # type: ignore
    from ._models import DataVersion  # type: ignore
    from ._models import DataVersionEntity  # type: ignore
    from ._models import DataVersionMutable  # type: ignore
    from ._models import DataViewSetResult  # type: ignore
    from ._models import Dataset  # type: ignore
    from ._models import DatasetDefinition  # type: ignore
    from ._models import DatasetDefinitionReference  # type: ignore
    from ._models import DatasetPath  # type: ignore
    from ._models import DatasetState  # type: ignore
    from ._models import DatasetV2  # type: ignore
    from ._models import EntityMetadata  # type: ignore
    from ._models import ErrorAdditionalInfo  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import HistogramBin  # type: ignore
    from ._models import HttpContent  # type: ignore
    from ._models import HttpMethod  # type: ignore
    from ._models import HttpRequestMessage  # type: ignore
    from ._models import HttpResponseMessage  # type: ignore
    from ._models import InnerErrorResponse  # type: ignore
    from ._models import KeyValuePairStringIEnumerable1  # type: ignore
    from ._models import LongRunningOperationResponse1LongRunningOperationResponseObject  # type: ignore
    from ._models import Moments  # type: ignore
    from ._models import PaginatedDataContainerEntityList  # type: ignore
    from ._models import PaginatedDataVersionEntityList  # type: ignore
    from ._models import PaginatedDatasetDefinitionList  # type: ignore
    from ._models import PaginatedDatasetList  # type: ignore
    from ._models import PaginatedDatasetV2List  # type: ignore
    from ._models import PaginatedStringList  # type: ignore
    from ._models import ProfileActionResult  # type: ignore
    from ._models import ProfileResult  # type: ignore
    from ._models import Quantiles  # type: ignore
    from ._models import RegisterExistingData  # type: ignore
    from ._models import RootError  # type: ignore
    from ._models import STypeCount  # type: ignore
    from ._models import SqlDataPath  # type: ignore
    from ._models import StoredProcedureParameter  # type: ignore
    from ._models import StringLengthCount  # type: ignore
    from ._models import TypeCount  # type: ignore
    from ._models import User  # type: ignore
    from ._models import ValueCount  # type: ignore

from ._azure_machine_learning_workspaces_enums import (
    DataflowType,
    FieldType,
    HttpStatusCode,
    HttpVersionPolicy,
    SType,
    StoredProcedureParameterType,
)

__all__ = [
    'ActionResult',
    'AssetId',
    'BatchDataUriResponse',
    'BatchGetResolvedURIs',
    'ColumnDefinition',
    'CreateUnregisteredInputData',
    'CreateUnregisteredOutputData',
    'DataCallRequest',
    'DataContainer',
    'DataContainerEntity',
    'DataContainerMutable',
    'DataField',
    'DataUriV2Response',
    'DataVersion',
    'DataVersionEntity',
    'DataVersionMutable',
    'DataViewSetResult',
    'Dataset',
    'DatasetDefinition',
    'DatasetDefinitionReference',
    'DatasetPath',
    'DatasetState',
    'DatasetV2',
    'EntityMetadata',
    'ErrorAdditionalInfo',
    'ErrorResponse',
    'HistogramBin',
    'HttpContent',
    'HttpMethod',
    'HttpRequestMessage',
    'HttpResponseMessage',
    'InnerErrorResponse',
    'KeyValuePairStringIEnumerable1',
    'LongRunningOperationResponse1LongRunningOperationResponseObject',
    'Moments',
    'PaginatedDataContainerEntityList',
    'PaginatedDataVersionEntityList',
    'PaginatedDatasetDefinitionList',
    'PaginatedDatasetList',
    'PaginatedDatasetV2List',
    'PaginatedStringList',
    'ProfileActionResult',
    'ProfileResult',
    'Quantiles',
    'RegisterExistingData',
    'RootError',
    'STypeCount',
    'SqlDataPath',
    'StoredProcedureParameter',
    'StringLengthCount',
    'TypeCount',
    'User',
    'ValueCount',
    'DataflowType',
    'FieldType',
    'HttpStatusCode',
    'HttpVersionPolicy',
    'SType',
    'StoredProcedureParameterType',
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


import json
import shutil
from pathlib import Path
from typing import Iterable, Optional

from azure.ai.ml.constants._common import DefaultOpenEncoding
from azure.ai.ml.entities import OnlineEndpoint
from azure.ai.ml.entities._load_functions import load_online_endpoint


class EndpointStub:
    """EndpointStub is a class for representing local endpoints which do not have deployments created under them yet.

    To maintain a catalog of local endpoints, it writes a yaml file with the endpoint specification to the user's
    machine in an idempotent, well-known location.
    """

    def create_or_update(self, endpoint: OnlineEndpoint) -> OnlineEndpoint:
        """Create or update a local endpoint.

        :param OnlineEndpoint endpoint: OnlineEndpoint entity to create or update.
        :return: The provided endpoint
        :rtype: OnlineEndpoint
        """
        self._create_endpoint_cache(endpoint=endpoint)
        return endpoint

    def get(self, endpoint_name: str) -> Optional[OnlineEndpoint]:
        """Get a local endpoint.

        :param str endpoint_name: Name of local endpoint to get.
        :return: The specified Online Endpoint
        :rtype: Optional[Endpoint]
        """
        endpoint_path = self._get_endpoint_cache_file(endpoint_name=endpoint_name)
        if endpoint_path.exists():
            return load_online_endpoint(source=endpoint_path)
        return None

    def list(self) -> Iterable[Path]:
        """List all local endpoints.

        :return: An iterable of paths to endpoints
        :rtype: Iterable[Path]
        """
        endpoints = []
        azureml_dir = self._get_inferencing_cache_dir()
        for endpoint_file in azureml_dir.glob("*/*.json"):
            endpoints.append(endpoint_file)
        return endpoints

    def delete(self, endpoint_name: str):
        """Delete a local endpoint.

        :param str endpoint_name: Name of local endpoint to delete.
        """
        build_directory = self._get_build_directory(endpoint_name=endpoint_name)
        shutil.rmtree(build_directory)

    def invoke(self):
        """Invoke a local endpoint.

        For an EndpointStub, it cannot invoke, so we return a helper message.

        :return: Invocation result
        :rtype: str
        """
        return (
            "This local endpoint does not have any deployments, so it cannot be invoked."
            "Please use 'az ml online-deployment create --local' before invoking."
        )

    def _create_endpoint_cache(self, endpoint: OnlineEndpoint) -> Path:
        """Create or update a local endpoint cache.

        :param OnlineEndpoint endpoint: OnlineEndpoint entity to create or update.
        :return: The endpoint cache path
        :rtype: Path
        """
        endpoint_cache_path = self._get_endpoint_cache_file(endpoint_name=str(endpoint.name))
        endpoint_metadata = json.dumps(endpoint.dump())
        endpoint_cache_path.write_text(endpoint_metadata, encoding=DefaultOpenEncoding.WRITE)
        return endpoint_cache_path

    def _get_endpoint_cache_file(self, endpoint_name: str) -> Path:
        """Get a local endpoint cache Path. Idempotent.

        :param str endpoint_name: Name of local endpoint to get local cache.
        :return: path to cached endpoint file.
        :rtype: Path
        """
        build_directory = self._create_build_directory(endpoint_name=endpoint_name)
        return Path(build_directory, f"{endpoint_name}.json")

    def _create_build_directory(self, endpoint_name: str) -> Path:
        """Create or update a local endpoint build directory.

        :param str endpoint_name: Name of local endpoint to get local directory.
        :return: path to endpoint build directory.
        :rtype: Path
        """
        build_directory = self._get_build_directory(endpoint_name=endpoint_name)
        build_directory.mkdir(parents=True, exist_ok=True)
        return build_directory

    def _get_build_directory(self, endpoint_name: str) -> Path:
        """Get a local endpoint build directory. Idempotent.

        :param str endpoint_name: Name of local endpoint to get local directory.
        :return: path to endpoint build directory.
        :rtype: Path
        """
        return Path(self._get_inferencing_cache_dir(), endpoint_name)

    @classmethod
    def _get_inferencing_cache_dir(cls) -> Path:
        """Get a local inferencing directory. Idempotent.

        :return: path to local inferencing cache directory.
        :rtype: Path
        """
        return Path(Path.home(), ".azureml", "inferencing")



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from marshmallow import post_load
from azure.ai.ml.constants._job.job import JobPriorityValues, JobTierNames
from azure.ai.ml._schema.core.fields import StringTransformedEnum
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta


class QueueSettingsSchema(metaclass=PatchedSchemaMeta):
    job_tier = StringTransformedEnum(
        allowed_values=JobTierNames.ALLOWED_NAMES,
    )
    priority = StringTransformedEnum(
        allowed_values=JobPriorityValues.ALLOWED_VALUES,
    )

    @post_load
    def make(self, data, **kwargs):  # pylint: disable=unused-argument
        from azure.ai.ml.entities import QueueSettings

        return QueueSettings(**data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from os import PathLike
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from marshmallow import INCLUDE

from azure.ai.ml._schema._data.mltable_metadata_schema import MLTableMetadataSchema
from azure.ai.ml._utils.utils import load_yaml
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY
from azure.ai.ml.entities._util import load_from_dict


class MLTableMetadataPath:
    type: str  # Literal["pattern", "file", "folder"]
    value: Optional[str]

    def __init__(self, *, pathDict: Dict):
        if pathDict.get("pattern", None):
            self.type = "pattern"
            self.value = pathDict.get("pattern")
        if pathDict.get("file", None):
            self.type = "file"
            self.value = pathDict.get("file")
        if pathDict.get("folder", None):
            self.type = "folder"
            self.value = pathDict.get("folder")


class MLTableMetadata:
    """MLTableMetadata for data assets.

    :param paths: List of paths which the MLTableMetadata refers to.
    :type paths: List[MLTableMetadataPath]
    :param transformations: Any transformations to be applied to the data referenced in paths.
    :type transformations: List[Any]
    :param base_path: Base path to resolve relative paths from.
    :type base_path: str
    """

    def __init__(
        self,
        *,
        paths: List[MLTableMetadataPath],
        transformations: Optional[List[Any]] = None,
        base_path: str,
        **_kwargs: Any,
    ):
        self.base_path = base_path
        self.paths = paths
        self.transformations = transformations

    @classmethod
    def load(
        cls,
        yaml_path: Union[PathLike, str],
        **kwargs: Any,
    ) -> "MLTableMetadata":
        """Construct an MLTable object from yaml file.

        :param yaml_path: Path to a local file as the source.
        :type yaml_path: PathLike | str

        :return: Constructed MLTable object.
        :rtype: MLTable
        """
        yaml_dict = load_yaml(yaml_path)
        return cls._load(yaml_data=yaml_dict, yaml_path=yaml_path, **kwargs)

    @classmethod
    def _load(
        cls,
        yaml_data: Optional[Dict],
        yaml_path: Optional[Union[PathLike, str]],
        **kwargs: Any,
    ) -> "MLTableMetadata":
        yaml_data = yaml_data or {}
        context = {
            BASE_PATH_CONTEXT_KEY: Path(yaml_path).parent if yaml_path else Path("./"),
        }
        res: MLTableMetadata = load_from_dict(MLTableMetadataSchema, yaml_data, context, "", unknown=INCLUDE, **kwargs)
        return res

    def _to_dict(self) -> Dict:
        res: dict = MLTableMetadataSchema(context={BASE_PATH_CONTEXT_KEY: "./"}, unknown=INCLUDE).dump(self)
        return res

    def referenced_uris(self) -> List[Optional[str]]:
        return [path.value for path in self.paths]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import json
import os.path
from typing import Callable, Dict, List

import pydash
import pytest
from devtools_testutils import AzureRecordedTestCase

from azure.ai.ml import MLClient, load_component
from azure.ai.ml._internal.entities import InternalComponent

from .._utils import PARAMETERS_TO_TEST


def create_component(
    client: MLClient,
    component_name: str,
    path="./tests/test_configs/components/helloworld_component.yml",
    params_override=None,
    is_anonymous=False,
):
    default_param_override = [{"name": component_name}]
    if params_override is None:
        params_override = default_param_override
    else:
        params_override += default_param_override

    command_component = load_component(
        source=path,
        params_override=params_override,
    )
    return client.components.create_or_update(command_component, is_anonymous=is_anonymous)


def load_registered_component(
    client: MLClient,
    component_name: str,
    component_version: str,
    omit_fields: List[str],
) -> Dict:
    """Load registered component from server and omit some fields."""
    component_entity = client.components.get(name=component_name, version=component_version)
    component_rest_object = component_entity._to_rest_object()
    return pydash.omit(component_rest_object.properties.component_spec, *omit_fields)


# previous bodiless_matcher fixture doesn't take effect because of typo, please add it in method level if needed


@pytest.mark.usefixtures(
    "recorded_test",
    "enable_internal_components",
    "mock_code_hash",
    "mock_asset_name",
    "mock_component_hash",
)
@pytest.mark.e2etest
@pytest.mark.pipeline_test
class TestComponent(AzureRecordedTestCase):
    @pytest.mark.parametrize(
        "yaml_path,inputs,runsettings_dict,pipeline_runsettings_dict",
        PARAMETERS_TO_TEST,
    )
    def test_component_create(
        self,
        client: MLClient,
        randstr: Callable[[], str],
        yaml_path: str,
        inputs: Dict,
        runsettings_dict: Dict,
        pipeline_runsettings_dict: Dict,
    ) -> None:
        component_name = randstr("component_name")
        component_resource = create_component(client, component_name, path=yaml_path)
        assert component_resource.name == component_name
        assert component_resource.code
        assert component_resource.creation_context

        component = client.components.get(component_name, component_resource.version)
        assert component_resource._to_dict() == component._to_dict()
        assert component_resource.creation_context

    @pytest.mark.parametrize(
        "yaml_path,inputs,runsettings_dict,pipeline_runsettings_dict",
        PARAMETERS_TO_TEST,
    )
    def test_component_load(
        self,
        client: MLClient,
        randstr: Callable[[], str],
        yaml_path: str,
        inputs: Dict,
        runsettings_dict: Dict,
        pipeline_runsettings_dict: Dict,
    ) -> None:
        omit_fields = ["id", "creation_context", "code", "name"]
        component_name = randstr("component_name")

        component_resource = create_component(client, component_name, path=yaml_path)
        loaded_dict = load_registered_component(client, component_name, component_resource.version, omit_fields)

        base_dir = "./tests/test_configs/internal"
        json_path = yaml_path.rsplit(".", 1)[0] + ".json"
        json_path = os.path.join(base_dir, "loaded_from_rest", os.path.relpath(json_path, base_dir))
        os.makedirs(os.path.dirname(json_path), exist_ok=True)
        if not os.path.isfile(json_path):
            with open(json_path, "w") as f:
                json.dump(loaded_dict, f, indent=2)
        with open(json_path, "r") as f:
            expected_dict = json.load(f)
            expected_dict["_source"] = "REMOTE.WORKSPACE.COMPONENT"

            # default value for datatransfer
            if expected_dict["type"] == "DataTransferComponent" and "datatransfer" not in expected_dict:
                expected_dict["datatransfer"] = {"allow_overwrite": "True"}

            # skip environment in arm string
            if "environment" in loaded_dict and isinstance(loaded_dict["environment"], str):
                omit_fields.append("environment")
            # TODO: check if loaded environment is expected to be an ordered dict
            assert pydash.omit(loaded_dict, *omit_fields) == pydash.omit(expected_dict, *omit_fields)

    def test_component_code_hash(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        yaml_path = "./tests/test_configs/internal/command-component-reuse/powershell_copy.yaml"
        expected_snapshot_id = "75c43313-4777-b2e9-fe3a-3b98cabfaa77"

        for component_name_key in ["component_name", "component_name2"]:
            component_name = randstr(component_name_key)
            component_resource: InternalComponent = create_component(client, component_name, path=yaml_path)
            assert component_resource.code.endswith(f"codes/{expected_snapshot_id}/versions/1")



import os.path
import re
import shutil
import tempfile
import uuid
from itertools import tee
from pathlib import Path
from typing import Callable

import pydash
import pytest
from devtools_testutils import AzureRecordedTestCase, is_live
from test_utilities.utils import assert_job_cancel, omit_with_wildcard, sleep_if_live

from azure.ai.ml import MLClient, MpiDistribution, load_component, load_environment
from azure.ai.ml._restclient.v2022_05_01.models import ListViewType
from azure.ai.ml._utils._arm_id_utils import is_ARM_id_for_resource
from azure.ai.ml.constants._assets import IPProtectionLevel
from azure.ai.ml.constants._common import (
    ANONYMOUS_COMPONENT_NAME,
    ARM_ID_PREFIX,
    PROVIDER_RESOURCE_ID_WITH_VERSION,
    AzureMLResourceType,
)
from azure.ai.ml.dsl._utils import _sanitize_python_variable_name
from azure.ai.ml.entities import CommandComponent, Component, PipelineComponent
from azure.ai.ml.entities._load_functions import load_code, load_job
from azure.core.exceptions import HttpResponseError
from azure.core.paging import ItemPaged

from .._util import _COMPONENT_TIMEOUT_SECOND
from ..unittests.test_component_schema import load_component_entity_from_rest_json


def create_component(
    client: MLClient,
    component_name: str,
    path="./tests/test_configs/components/helloworld_component.yml",
    params_override=None,
    is_anonymous=False,
    **kwargs,
):
    default_param_override = [{"name": component_name}]
    if params_override is None:
        params_override = default_param_override
    else:
        params_override += default_param_override

    command_component = load_component(
        source=path,
        params_override=params_override,
    )
    return client.components.create_or_update(command_component, is_anonymous=is_anonymous, **kwargs)


@pytest.fixture
def torch_distribution():
    def create_torch_distribution(has_strs: bool = False):
        # service returns component with stringified integer values
        # need to do comparison with object returned by service with strings
        # comparison with object in SDK is done with integer values
        if not has_strs:
            return {"type": "pytorch", "process_count_per_instance": 4}
        return {"type": "pytorch", "process_count_per_instance": "4"}

    return create_torch_distribution


@pytest.fixture
def tensorflow_distribution():
    def create_tensorflow_distribution(has_strs: bool = False):
        # service returns component with stringified integer values
        # need to do comparison with object returned by service with strings
        # comparison with object in SDK is done with integer values
        if not has_strs:
            return {
                "type": "tensorflow",
                "parameter_server_count": 1,
                "worker_count": 2,
            }
        return {
            "type": "tensorflow",
            "parameter_server_count": "1",
            "worker_count": "2",
        }

    return create_tensorflow_distribution


# previous bodiless_matcher fixture doesn't take effect because of typo, please add it in method level if needed


def assert_component_basic_workflow(
    client: MLClient,
    randstr: Callable[[str], str],
    path: str,
    expected_dict: dict,
    omit_fields: list,
    recorded_component_name: str,
):
    component_name = randstr(recorded_component_name)

    # create a component
    component_resource = create_component(client, component_name, path=path)
    actual_dict = pydash.omit(dict(component_resource._to_dict()), omit_fields)
    assert actual_dict == expected_dict
    assert component_resource.name == component_name
    assert component_resource.creation_context

    # get back
    component = client.components.get(component_name, component_resource.version)
    assert component_resource._to_dict() == component._to_dict()
    assert component_resource.creation_context

    # create a new version
    params_override = [{"name": component_name}, {"version": 2}]
    command_component = load_component(
        source=path,
        params_override=params_override,
    )
    component_resource = client.components.create_or_update(command_component)
    assert component_resource.version == "2"


@pytest.mark.e2etest
@pytest.mark.timeout(_COMPONENT_TIMEOUT_SECOND)
@pytest.mark.usefixtures(
    "recorded_test",
    "mock_code_hash",
    "mock_asset_name",
    "mock_component_hash",
    "enable_environment_id_arm_expansion",
)
@pytest.mark.pipeline_test
class TestComponent(AzureRecordedTestCase):
    def test_command_component(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        expected_dict = {
            "$schema": "https://azuremlschemas.azureedge.net/development/commandComponent.schema.json",
            "command": "echo Hello World & echo $[[${{inputs.component_in_number}}]] & echo "
            "${{inputs.component_in_path}} & echo "
            "${{outputs.component_out_path}} > "
            "${{outputs.component_out_path}}/component_in_number",
            "description": "This is the basic command component",
            "display_name": "CommandComponentBasic",
            "inputs": {
                "component_in_number": {
                    "default": "10.99",
                    "description": "A number",
                    "optional": True,
                    "type": "number",
                },
                "component_in_path": {"description": "A path", "type": "uri_folder", "optional": False},
            },
            "is_deterministic": True,
            "outputs": {"component_out_path": {"type": "uri_folder"}},
            "resources": {"instance_count": 1},
            "tags": {"owner": "sdkteam", "tag": "tagvalue"},
            "type": "command",
            "version": "0.0.1",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/helloworld_component.yml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id", "code", "environment"],
            recorded_component_name="component_name",
        )

    def test_parallel_component(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        expected_dict = {
            "$schema": "http://azureml/sdk-2-0/ParallelComponent.json",
            "description": "parallel component for batch score",
            "display_name": "BatchScore",
            "error_threshold": 10,
            "input_data": "${{inputs.score_input}}",
            "inputs": {
                "label": {
                    "description": "Other reference data for batch scoring, " "e.g. labels.",
                    "type": "uri_file",
                    "optional": False,
                },
                "score_input": {
                    "description": "The data to be split and scored in " "parallel.",
                    "type": "mltable",
                    "optional": False,
                },
                "score_model": {"description": "The model for batch score.", "type": "custom_model", "optional": False},
            },
            "is_deterministic": True,
            "max_concurrency_per_instance": 12,
            "mini_batch_error_threshold": 5,
            "logging_level": "INFO",
            "mini_batch_size": "10240",
            "outputs": {"scored_result": {"type": "mltable"}, "scoring_summary": {"type": "uri_file"}},
            "retry_settings": {"max_retries": 10, "timeout": 3},
            "type": "parallel",
            "version": "1.0.0",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/basic_parallel_component_score.yml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id", "task"],
            recorded_component_name="component_name",
        )

    def test_automl_component(self, client: MLClient, registry_client: MLClient, randstr: Callable[[str], str]) -> None:
        expected_component_dict = {
            "description": "Component that executes an AutoML Classification task model training in a pipeline.",
            "version": "1.0",
            "$schema": "http://azureml/sdk-2-0/AutoMLComponent.json",
            "display_name": "AutoML Classification",
            "type": "automl",
            "task": "classification",
        }
        omit_fields = [
            "name",
            "creation_context",
            "id",
            # still passing these fields to backend and let backend decide if they want to respect that
            "inputs",
            "outputs",
            "is_deterministic",
        ]
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/automl/classification.yaml",
            expected_dict=expected_component_dict,
            omit_fields=omit_fields,
            recorded_component_name="workspace_component_name",
        )

        assert_component_basic_workflow(
            client=registry_client,
            randstr=randstr,
            path="./tests/test_configs/components/automl/classification.yaml",
            expected_dict=expected_component_dict,
            omit_fields=omit_fields,
            recorded_component_name="registry_component_name",
        )

    def test_spark_component(self, client: MLClient, randstr: Callable[[], str]) -> None:
        expected_dict = {
            "$schema": "https://azuremlschemas.azureedge.net/latest/sparkComponent.schema.json",
            "args": "--file_input ${{inputs.file_input}} --output ${{outputs.output}}",
            "conf": {
                "spark.driver.cores": 2,
                "spark.driver.memory": "1g",
                "spark.executor.cores": 1,
                "spark.executor.instances": 1,
                "spark.executor.memory": "1g",
            },
            "description": "Aml Spark dataset test module",
            "display_name": "Aml Spark dataset test module",
            "entry": {"file": "kmeans_example.py"},
            "inputs": {"file_input": {"type": "uri_file", "optional": False}},
            "is_deterministic": True,
            "outputs": {"output": {"type": "uri_folder"}},
            "type": "spark",
            "version": "1",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/spark_component/component.yml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id", "code", "environment"],
            recorded_component_name="spark_component_name",
        )

    def test_datatransfer_copy_urifolder_component(self, client: MLClient, randstr: Callable[[], str]) -> None:
        expected_dict = {
            "$schema": "http://azureml/sdk-2-0/DataTransferComponent.json",
            "data_copy_mode": "merge_with_overwrite",
            "display_name": "Data Transfer Component copy-files",
            "type": "data_transfer",
            "task": "copy_data",
            "inputs": {"folder1": {"type": "uri_folder", "optional": False}},
            "outputs": {"output_folder": {"type": "uri_folder"}},
            "is_deterministic": True,
            "version": "1",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/data_transfer/copy_files.yaml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id"],
            recorded_component_name="datatransfer_copy_urifolder",
        )

    def test_datatransfer_copy_urifile_component(self, client: MLClient, randstr: Callable[[], str]) -> None:
        expected_dict = {
            "$schema": "http://azureml/sdk-2-0/DataTransferComponent.json",
            "data_copy_mode": "fail_if_conflict",
            "display_name": "Data Transfer Component copy uri files",
            "type": "data_transfer",
            "task": "copy_data",
            "inputs": {"folder1": {"type": "uri_file", "optional": False}},
            "outputs": {"output_folder": {"type": "uri_file"}},
            "is_deterministic": True,
            "version": "1",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/data_transfer/copy_uri_files.yaml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id"],
            recorded_component_name="datatransfer_copy_urifile",
        )

    def test_datatransfer_copy_2urifolder_component(self, client: MLClient, randstr: Callable[[], str]) -> None:
        expected_dict = {
            "$schema": "http://azureml/sdk-2-0/DataTransferComponent.json",
            "display_name": "Data Transfer Component merge-files",
            "type": "data_transfer",
            "data_copy_mode": "merge_with_overwrite",
            "task": "copy_data",
            "inputs": {
                "folder1": {"type": "uri_folder", "optional": False},
                "folder2": {"type": "uri_folder", "optional": False},
            },
            "outputs": {"output_folder": {"type": "uri_folder"}},
            "is_deterministic": True,
            "version": "1",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/data_transfer/merge_files.yaml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id"],
            recorded_component_name="datatransfer_copy_2urifolder",
        )

    def test_datatransfer_copy_mixtype_component(self, client: MLClient, randstr: Callable[[], str]) -> None:
        expected_dict = {
            "$schema": "http://azureml/sdk-2-0/DataTransferComponent.json",
            "display_name": "Data Transfer Component merge mix type files",
            "type": "data_transfer",
            "data_copy_mode": "merge_with_overwrite",
            "task": "copy_data",
            "inputs": {
                "input1": {"type": "uri_file", "optional": False},
                "input2": {"type": "uri_file", "optional": False},
                "input3": {"type": "mltable", "optional": False},
            },
            "outputs": {"output_folder": {"type": "uri_folder"}},
            "is_deterministic": True,
            "version": "1",
        }
        assert_component_basic_workflow(
            client=client,
            randstr=randstr,
            path="./tests/test_configs/components/data_transfer/merge_mixtype_files.yaml",
            expected_dict=expected_dict,
            omit_fields=["name", "creation_context", "id"],
            recorded_component_name="datatransfer_copy_mixtype",
        )

    @pytest.mark.parametrize(
        "component_path",
        [
            "type_contract/mltable.yml",
            "type_contract/mlflow_model.yml",
            "type_contract/custom_model.yml",
            "type_contract/path.yml",
            "input_types_component.yml",
        ],
    )
    def test_command_component_create_input_output_types(
        self, client: MLClient, randstr: Callable[[str], str], component_path: str
    ) -> None:
        component_name = randstr("component_name")
        params_override = [{"name": component_name}]
        component_entity = load_component(
            source="./tests/test_configs/components/{}".format(component_path),
            params_override=params_override,
        )
        target_entity = client.components.create_or_update(component_entity)
        target_entity._creation_context = None
        target_entity.resources = None
        component_entity._creation_context = None
        assert target_entity.id
        # server side will remove \n from the code now. Skip them given it's not targeted to check in this test
        # server side will return optional False for optional None input
        omit_fields = ["id", "command", "environment", "inputs.*.optional", "properties"]
        assert omit_with_wildcard(component_entity._to_dict(), *omit_fields) == omit_with_wildcard(
            target_entity._to_dict(), *omit_fields
        )

    def test_command_component_with_code(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_name = randstr("component_name")

        component_resource = create_component(
            client,
            component_name,
            path="./tests/test_configs/components/basic_component_code_local_path.yml",
        )
        assert component_resource.name == component_name
        # make sure code is created
        assert component_resource.code
        assert is_ARM_id_for_resource(component_resource.code)

    def test_command_component_with_pathlike_as_code(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_name = randstr("component_name")

        component = load_component(source="./tests/test_configs/components/basic_component_code_local_path.yml")
        from pathlib import Path

        component.name = component_name
        component.code = Path(component.code)

        component_resource = client.components.create_or_update(component)
        assert component_resource.name == component_name
        # make sure code is created
        assert component_resource.code
        assert is_ARM_id_for_resource(component_resource.code)

    def test_component_list(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_name = randstr("component name")

        component_resource = create_component(client, component_name)
        assert component_resource.name == component_name

        # list component containers
        component_containers = client.components.list()
        assert isinstance(component_containers.next(), Component)

        # there might be delay so getting the latest version immediately after creation might get wrong result
        sleep_if_live(5)

        # list component versions
        components = client.components.list(name=component_name)
        # creating a copy of iterator
        components, components_copy = tee(components)
        # iterating the whole iterable object to find the number of elements. Not using list.
        assert sum(1 for e in components_copy) == 1
        component = next(iter(components), None)
        assert isinstance(component, Component)

    def test_component_update(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        # create a component
        component_name = randstr("component_name")

        component_resource = create_component(client, component_name)
        assert component_resource.id
        rest_path = "./tests/test_configs/components/helloworld_component_rest.json"
        target_entity = load_component_entity_from_rest_json(rest_path)
        assert target_entity._source == "REMOTE.WORKSPACE.COMPONENT"
        component_dict = pydash.omit(
            dict(component_resource._to_dict()),
            "code",
            "environment",
            "name",
            "creation_context",
            "resources",
            "id",
            "inputs.component_in_path.optional",  # backend will return component inputs as optional:False
        )
        expected_dict = pydash.omit(
            dict(target_entity._to_dict()),
            "code",
            "environment",
            "name",
            "creation_context",
            "id",
        )
        assert component_dict == expected_dict

        # create component again with new description and display name
        # TODO: test update tags when supported
        description = "Updated description"
        display_name = "UpdatedComponent"
        params_override = [{"description": description}, {"display_name": display_name}]
        component_entity = Component._load(
            data=component_resource._to_dict(),
            yaml_path="./tests/test_configs/components/helloworld_component.yml",
            params_override=params_override,
        )
        component_resource = client.components.create_or_update(component_entity)
        assert component_resource.description == description
        assert component_resource.display_name == display_name

    @pytest.mark.disable_mock_code_hash
    @pytest.mark.skipif(condition=not is_live(), reason="reuse test, target to verify service-side behavior")
    def test_component_create_twice_same_code_arm_id(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_path = "./tests/test_configs/components/component_for_reuse_test/component.yml"
        component_name = randstr("component_name")
        # create a component
        component_resource1 = create_component(client, component_name, path=component_path)
        # create again
        component_resource2 = create_component(client, component_name, path=component_path)
        # the code arm id should be the same
        assert component_resource1.code == component_resource2.code

    @pytest.mark.skipif(condition=not is_live(), reason="non-deterministic upload fails in playback on CI")
    def test_component_update_code(self, client: MLClient, randstr: Callable[[str], str], tmp_path: Path) -> None:
        component_name = randstr("component_name")
        path = "./tests/test_configs/components/basic_component_code_local_path.yml"

        # create a component
        create_component(client, component_name, path=path)

        # create again with different code and check error msg
        code_path = tmp_path / "code.yml"
        code_path.write_text(
            f"""
        name: {component_name}
        version: 1
        path: ."""
        )
        new_code = load_code(source=code_path)
        code_resource = client._code.create_or_update(new_code)
        params_override = [{"name": component_name, "code": ARM_ID_PREFIX + code_resource.id}]
        command_component = load_component(
            source=path,
            params_override=params_override,
        )
        with pytest.raises(HttpResponseError):
            client.components.create_or_update(command_component)

    @pytest.mark.disable_mock_code_hash
    def test_component_create_default_code(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        # step2: test component without code
        component_name = randstr("component_name")
        component_resource1 = create_component(client, component_name)

        # create again with updated tags
        description = "Updated description"
        display_name = "UpdatedComponent"
        params_override = [{"description": description}, {"display_name": display_name}]
        component_resource2 = create_component(client, component_name, params_override=params_override)

        # the code arm id should be the same
        assert component_resource1.code is None
        assert component_resource1.code == component_resource2.code
        assert component_resource2.description == description
        assert component_resource2.display_name == display_name

    @pytest.mark.disable_mock_code_hash
    def test_mpi_component(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_name = randstr("component_name")
        # Test mpi distribution
        params_override = [{"name": component_name}]
        component_entity = load_component(
            source="./tests/test_configs/components/helloworld_component_mpi.yml",
            params_override=params_override,
        )
        # additional fields added_property in distribution and resources are removed.
        mpi_distribution = {"type": "mpi", "process_count_per_instance": 1}
        assert mpi_distribution == component_entity.distribution.__dict__
        mpi_component_resource = client.components.create_or_update(component_entity)
        assert isinstance(mpi_component_resource.distribution, MpiDistribution)
        component_entity = CommandComponent(
            name=randstr("new_name"),
            display_name="CommandComponentMpi",
            description="This is the mpi command component",
            tags={"tag": "tagvalue", "owner": "sdkteam"},
            inputs={
                "component_in_number": {
                    "description": "A number",
                    "type": "number",
                    "default": 10.99,
                },
                "component_in_path": {"description": "A path", "type": "uri_folder"},
            },
            outputs={"component_out_path": {"type": "uri_folder"}},
            command="echo Hello World & echo ${{inputs.component_in_number}} & echo ${{inputs.component_in_path}} "
            "& echo ${{outputs.component_out_path}}",
            environment="AzureML-sklearn-1.0-ubuntu20.04-py38-cpu:33",
            distribution=MpiDistribution(
                process_count_per_instance=1,
                # No affect because Mpi object does not allow extra fields
                added_property=7,
            ),
            instance_count=2,
        )
        mpi_entity = client.components.create_or_update(component_entity)

        component_dict = pydash.omit(
            mpi_entity._to_dict(),
            "$schema",
            "id",
            "name",
            "creation_context",
        )
        yaml_component_dict = pydash.omit(
            mpi_component_resource._to_dict(),
            "$schema",
            "id",
            "name",
            "creation_context",
        )
        assert component_dict == yaml_component_dict

    def test_pytorch_component(
        self, client: MLClient, randstr: Callable[[str], str], torch_distribution: Callable[[bool], dict]
    ) -> None:
        component_name = randstr("component_name")
        # Test mpi distribution
        params_override = [{"name": component_name}]
        # Test torch distribution
        component_entity = load_component(
            source="./tests/test_configs/components/helloworld_component_pytorch.yml",
            params_override=params_override,
        )
        # additional fields added_property in distribution and resources are removed.
        torch_resources = {"instance_count": 2, "properties": {}}
        assert component_entity.distribution.__dict__ == torch_distribution()
        assert component_entity.resources._to_rest_object().as_dict() == torch_resources
        torch_component_resource = client.components.create_or_update(component_entity)
        assert torch_component_resource.resources._to_rest_object().as_dict() == torch_resources
        assert torch_component_resource.distribution.__dict__ == torch_distribution(has_strs=True)

    def test_tensorflow_component(
        self, client: MLClient, randstr: Callable[[str], str], tensorflow_distribution: Callable[[bool], dict]
    ) -> None:
        component_name = randstr("component_name")
        # Test mpi distribution
        params_override = [{"name": component_name}]
        # Test tensorflow distribution
        component_entity = load_component(
            source="./tests/test_configs/components/helloworld_component_tensorflow.yml",
            params_override=params_override,
        )
        assert component_entity.distribution.__dict__ == tensorflow_distribution()
        tensorflow_component_resource = client.components.create_or_update(component_entity)
        assert tensorflow_component_resource.distribution.__dict__ == tensorflow_distribution(has_strs=True)

    def test_command_component_create_autoincrement(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_name = randstr("component_name")
        params_override = [{"name": component_name}]
        path = "./tests/test_configs/components/component_no_version.yml"
        command_component = load_component(source=path, params_override=params_override)

        assert command_component.version is None
        assert command_component._auto_increment_version

        created_command_component = client.components.create_or_update(command_component)
        assert created_command_component.version == "1"
        assert created_command_component._auto_increment_version is False

        next_component_asset = client.components.create_or_update(command_component)
        next_version_regex = re.compile(r"\d{4}-\d{2}-\d{2}-\d{2}-\d{2}-\d{2}-\d{7}")
        assert next_version_regex.match(next_component_asset.version)
        assert next_component_asset._auto_increment_version is False

    @pytest.mark.disable_mock_code_hash
    @pytest.mark.skipif(condition=not is_live(), reason="reuse test, target to verify service-side behavior")
    def test_anonymous_component_reuse(self, client: MLClient, variable_recorder) -> None:
        # component with different name will be created as different instance;
        # therefore component reuse will not work as component name differs

        # component without code
        component_name_1 = variable_recorder.get_or_record("component_name_1", str(uuid.uuid4()))
        component_name_2 = variable_recorder.get_or_record("component_name_2", str(uuid.uuid4()))
        component_resource1 = create_component(
            client, _sanitize_python_variable_name(component_name_1), is_anonymous=True
        )
        component_resource2 = create_component(
            client, _sanitize_python_variable_name(component_name_2), is_anonymous=True
        )
        assert component_resource1.environment == component_resource2.environment
        assert component_resource1.code == component_resource2.code

        # component with inline code and inline environment
        path = "./tests/test_configs/components/helloworld_component_no_paths.yml"
        component_resource1 = create_component(
            client,
            _sanitize_python_variable_name(str(uuid.uuid4())),
            path=path,
            is_anonymous=True,
        )
        component_resource2 = create_component(
            client,
            _sanitize_python_variable_name(str(uuid.uuid4())),
            path=path,
            is_anonymous=True,
        )
        assert component_resource1.environment == component_resource2.environment
        assert component_resource1.code == component_resource2.code

    def test_command_component_dependency_label_resolution(
        self, client: MLClient, randstr: Callable[[str], str]
    ) -> None:
        """Checks that dependencies of the form azureml:name@label are resolved to a version"""
        environment_name = randstr("environment_name")
        environment_versions = ["foo", "bar"]

        for version in environment_versions:
            client.environments.create_or_update(
                load_environment(
                    "./tests/test_configs/environment/environment_conda_inline.yml",
                    params_override=[{"name": environment_name}, {"version": version}],
                )
            )
        sleep_if_live(10)

        created_component = create_component(
            client,
            randstr("component_name"),
            params_override=[{"environment": f"azureml:{environment_name}@latest"}],
        )
        expected = PROVIDER_RESOURCE_ID_WITH_VERSION.format(
            client._operation_scope._subscription_id,
            client._operation_scope._resource_group_name,
            client._operation_scope.workspace_name,
            AzureMLResourceType.ENVIRONMENT,
            environment_name,
            environment_versions[-1],
        )

        assert created_component.environment == expected

    def test_command_component_get_latest_label(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        name = randstr("name")
        versions = ["foo", "bar", "baz", "foobar"]

        for version in versions:
            create_component(client, name, params_override=[{"version": version}])
            sleep_if_live(5)
            assert client.components.get(name, label="latest").version == version

    def test_anonymous_registration_from_load_component(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        command_component = load_component(source="./tests/test_configs/components/helloworld_component.yml")
        component_resource = client.components.create_or_update(command_component, is_anonymous=True)
        assert component_resource.name == command_component.name
        assert component_resource.version == command_component.version

        anonymous_name, _, anonymous_version = component_resource.id.split("/")[-3:]
        assert anonymous_name == ANONYMOUS_COMPONENT_NAME
        # version calculation has been moved to the server side

        component = client.components.get(anonymous_name, anonymous_version)
        # TODO 1807731: enable this check after server-side fix
        omit_fields = ["creation_context"]
        assert pydash.omit(component_resource._to_dict(), *omit_fields) == pydash.omit(
            component._to_dict(), *omit_fields
        )
        assert component.name == command_component.name
        assert component.version == command_component.version
        assert component._source == "REMOTE.WORKSPACE.COMPONENT"

    def test_component_archive_restore_version(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        name = randstr("namee")
        versions = ["1", "2"]
        version_archived = versions[0]
        for version in versions:
            params_override = [{"version": version}]
            create_component(client, name, params_override=params_override)

        def get_component_list():
            # Wait for list index to update before calling list
            if is_live():
                sleep_if_live(30)
            component_list = client.components.list(name=name, list_view_type=ListViewType.ACTIVE_ONLY)
            return [c.version for c in component_list]

        assert version_archived in get_component_list()
        client.components.archive(name=name, version=version_archived)
        assert version_archived not in get_component_list()
        client.components.restore(name=name, version=version_archived)
        assert version_archived in get_component_list()

    @pytest.mark.skipif(condition=not is_live(), reason="target to verify service-side behavior")
    def test_component_archive_restore_container(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        name = randstr("name")
        create_component(client, name)

        def get_component_list():
            # Wait for list index to update before calling list
            sleep_if_live(30)
            component_list = client.components.list(list_view_type=ListViewType.ACTIVE_ONLY)
            return [c.name for c in component_list]

        assert name in get_component_list()
        client.components.archive(name=name)
        assert name not in get_component_list()
        client.components.restore(name=name)
        assert name in get_component_list()

    def test_entity_command_component_create(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_name = randstr("component_name")
        component = CommandComponent(
            name=component_name,
            display_name="CommandComponentBasic",
            description="This is the basic command component",
            version="3",
            tags={"tag": "tagvalue", "owner": "sdkteam"},
            outputs={"component_out_path": {"type": "path"}},
            command="echo Hello World",
            environment="AzureML-sklearn-1.0-ubuntu20.04-py38-cpu:33",
            code="./tests/test_configs/components/helloworld_components_with_env",
        )
        component_resource = client.components.create_or_update(component)
        assert component_resource.version == "3"

    @pytest.mark.skipif(
        condition=not is_live(),
        reason="registry test, may fail in playback mode during retrieving registry client",
    )
    def test_component_create_get_list_from_registry(
        self, pipelines_registry_client: MLClient, randstr: Callable[[str], str]
    ) -> None:
        component_name = randstr("component_name")

        component_resource = create_component(pipelines_registry_client, component_name)
        assert component_resource.name == component_name
        assert component_resource.code
        assert component_resource.creation_context

        component_get = pipelines_registry_client.components.get(component_name, component_resource.version)
        assert component_resource._to_dict() == component_get._to_dict()
        assert component_resource.creation_context
        assert component_resource._source == "REMOTE.REGISTRY"

        components = pipelines_registry_client.components.list(name=component_name)
        assert isinstance(components, ItemPaged)
        test_component = next(iter(components), None)
        assert isinstance(test_component, Component)

    def test_simple_pipeline_component_create(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_path = "./tests/test_configs/components/helloworld_inline_pipeline_component.yml"

        component = load_component(
            source=component_path,
        )
        # Assert binding on compute not changed after resolve dependencies
        client.components._resolve_dependencies_for_pipeline_component_jobs(
            component, resolver=client.components._orchestrators.get_asset_arm_id
        )
        assert component.jobs["component_a_job"].compute == "${{parent.inputs.node_compute}}"
        # Assert E2E
        component_name = randstr("component_name")
        rest_pipeline_component = create_component(
            client,
            component_name=component_name,
            path=component_path,
        )
        assert rest_pipeline_component is not None
        component_dict = pydash.omit(
            dict(rest_pipeline_component._to_dict()),
            "name",
            "creation_context",
            "jobs",
            "id",
        )
        expected_dict = {
            "description": "This is the basic pipeline component",
            "tags": {"tag": "tagvalue", "owner": "sdkteam"},
            "version": "1",
            "$schema": "https://azuremlschemas.azureedge.net/development/pipelineComponent.schema.json",
            "display_name": "Hello World Pipeline Component",
            "is_deterministic": False,
            "inputs": {
                "component_in_path": {"type": "uri_folder", "description": "A path", "optional": False},
                "component_in_number": {
                    "type": "number",
                    "optional": True,
                    "default": "10.99",
                    "description": "A number",
                },
                # The azureml: prefix has been resolve and removed by service
                "node_compute": {"type": "string", "default": "cpu-cluster", "optional": False},
            },
            "type": "pipeline",
        }
        assert component_dict == expected_dict
        rest_dict = rest_pipeline_component._to_dict()
        if "jobs" in rest_dict:
            # below line is expected to raise KeyError in live test,
            # it will pass after related changes deployed to canary
            jobs_dict = rest_dict["jobs"]
            # Assert full componentId extra azureml prefix has been removed and parsed to versioned arm id correctly.
            assert "azureml:azureml_anonymous" in jobs_dict["component_a_job"]["component"]
            assert jobs_dict["component_a_job"]["type"] == "command"
            # Assert component show result
            rest_pipeline_component2 = client.components.get(name=component_name, version="1")
            jobs_dict2 = rest_pipeline_component2._to_dict()["jobs"]
            assert jobs_dict == jobs_dict2

    def test_helloworld_nested_pipeline_component(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        component_path = "./tests/test_configs/components/helloworld_nested_pipeline_component.yml"
        rest_pipeline_component = create_component(
            client,
            component_name=randstr("component_name"),
            path=component_path,
        )
        assert rest_pipeline_component is not None
        component_dict = pydash.omit(
            dict(rest_pipeline_component._to_dict()),
            "name",
            "creation_context",
            # jobs not returned now
            "jobs",
            "id",
        )
        expected_dict = {
            "description": "This is the basic pipeline component",
            "tags": {"tag": "tagvalue", "owner": "sdkteam"},
            "version": "1",
            "$schema": "https://azuremlschemas.azureedge.net/development/pipelineComponent.schema.json",
            "display_name": "Hello World Pipeline Component",
            "is_deterministic": False,
            "inputs": {
                "component_in_path": {
                    "type": "uri_folder",
                    "description": "A path for pipeline component",
                    "optional": False,
                },
                "component_in_number": {
                    "type": "number",
                    "optional": True,
                    "default": "10.99",
                    "description": "A number for pipeline component",
                },
            },
            "outputs": {"nested_output": {"type": "uri_folder"}, "nested_output2": {"type": "uri_folder"}},
            "type": "pipeline",
        }
        assert component_dict == expected_dict

    @pytest.mark.usefixtures("mock_set_headers_with_user_aml_token")
    def test_create_pipeline_component_from_job(self, client: MLClient, randstr: Callable[[str], str]):
        params_override = [{"name": randstr("component_name_0")}]
        pipeline_job = load_job(
            "./tests/test_configs/dsl_pipeline/pipeline_with_pipeline_component/pipeline.yml",
            params_override=params_override,
        )
        job = assert_job_cancel(pipeline_job, client)
        name = randstr("component_name_1")
        component = PipelineComponent(name=name, source_job_id=job.id)
        rest_component = client.components.create_or_update(component)
        assert rest_component.name == name

    @pytest.mark.skipif(condition=not is_live(), reason="registry test, target to verify service-side behavior")
    def test_component_with_default_label(
        self,
        client: MLClient,
        randstr: Callable[[str], str],
    ) -> None:
        yaml_path: str = "./tests/test_configs/components/helloworld_component.yml"
        component_name = randstr("component_name")

        create_component(client, component_name, path=yaml_path)

        sleep_if_live(5)  # sleep 5 seconds to wait for index service update

        target_component = client.components.get(component_name, label="latest")

        for default_component in [
            client.components.get(component_name),
            client.components.get(component_name, label="default"),
        ]:
            expected_component_dict = target_component._to_dict()
            default_component_dict = default_component._to_dict()
            assert pydash.omit(default_component_dict, "id") == pydash.omit(expected_component_dict, "id")

            assert default_component.id.endswith(f"/components/{component_name}/labels/default")

            node = default_component()
            assert node._to_rest_object()["componentId"] == default_component.id

    def test_command_component_with_properties_e2e_flow(self, client: MLClient, randstr: Callable[[str], str]) -> None:
        command_component = load_component(
            source="./tests/test_configs/components/helloworld_component_with_properties.yml",
        )
        expected_dict = {
            "$schema": "https://azuremlschemas.azureedge.net/development/commandComponent.schema.json",
            "_source": "YAML.COMPONENT",
            "command": "echo Hello World & echo $[[${{inputs.component_in_number}}]] & "
            "echo ${{inputs.component_in_path}} & echo "
            "${{outputs.component_out_path}} > "
            "${{outputs.component_out_path}}/component_in_number",
            "description": "This is the basic command component",
            "display_name": "CommandComponentBasic",
            "inputs": {
                "component_in_number": {
                    "default": "10.99",
                    "description": "A number",
                    "optional": True,
                    "type": "number",
                },
                "component_in_path": {"description": "A path", "type": "uri_folder"},
            },
            "is_deterministic": True,
            "outputs": {"component_out_path": {"type": "uri_folder"}},
            "properties": {"azureml.pipelines.dynamic": "true"},
            "tags": {"owner": "sdkteam", "tag": "tagvalue"},
            "type": "command",
        }
        omit_fields = ["name", "creation_context", "id", "code", "environment", "version"]
        rest_component = pydash.omit(
            command_component._to_rest_object().as_dict()["properties"]["component_spec"],
            omit_fields,
        )

        assert rest_component == expected_dict

        from_rest_component = client.components.create_or_update(command_component, is_anonymous=True)

        previous_dict = pydash.omit(command_component._to_dict(), omit_fields)
        current_dict = pydash.omit(from_rest_component._to_dict(), omit_fields)
        # TODO(2037030): verify when backend ready
        # assert previous_dict == current_dict

    @pytest.mark.skip(
        reason="TODO (2349965): Message: User/tenant/subscription is not allowed to access registry UnsecureTest-hello-world"
    )
    @pytest.mark.usefixtures("enable_private_preview_schema_features")
    def test_ipp_component_create(self, ipp_registry_client: MLClient, randstr: Callable[[str], str]):
        component_path = "./tests/test_configs/components/component_ipp.yml"
        command_component = load_component(source=component_path)
        from_rest_component = create_component(
            ipp_registry_client,
            component_name=randstr("component_name"),
            path=component_path,
        )

        assert from_rest_component._intellectual_property
        assert from_rest_component._intellectual_property == command_component._intellectual_property

        assert from_rest_component.inputs["training_data"]._intellectual_property
        assert (
            from_rest_component.inputs["training_data"]._intellectual_property
            == command_component.inputs["training_data"]._intellectual_property
        )

        assert from_rest_component.inputs["base_model"]._intellectual_property
        assert from_rest_component.inputs["base_model"]._intellectual_property.protection_level == IPProtectionLevel.ALL

        assert from_rest_component.outputs["model_output_not_ipp"]._intellectual_property

        assert (
            from_rest_component.outputs["model_output_not_ipp"]._intellectual_property
            == command_component.outputs["model_output_not_ipp"]._intellectual_property
        )

        assert from_rest_component.outputs["model_output_ipp"]._intellectual_property
        assert (
            from_rest_component.outputs["model_output_ipp"]._intellectual_property
            == command_component.outputs["model_output_ipp"]._intellectual_property
        )

    def test_create_component_skip_if_no_change(self, client: MLClient, randstr):
        component_operation = client._operation_container.all_operations[AzureMLResourceType.COMPONENT]
        component_name = "test_skip_if_no_change"
        try:
            default_component = component_operation.get(name=component_name)
        except Exception:
            default_component = None
        default_version = default_component.version if default_component else "1"
        # update  default component by current local component data.
        default_component = create_component(client, component_name=component_name, version=default_version)

        # test component has no change and use skip_if_no_change parameter
        new_version = randstr("component_version")
        new_component = create_component(
            client, component_name=component_name, version=new_version, skip_if_no_change=True
        )
        assert default_component._get_component_hash(
            keys_to_omit=["creation_context"]
        ) == new_component._get_component_hash(keys_to_omit=["creation_context"])

        # test component has change and use skip_if_no_change parameter
        new_version = randstr("component_version")
        params_override = [
            {"description": "description_{0}".format(new_version)},
            {"display_name": "display_name_{0}".format(new_version)},
            {"tags": {"tags": "tags_{0}".format(new_version)}},
        ]
        new_component = create_component(
            client,
            component_name=component_name,
            version=new_version,
            params_override=params_override,
            skip_if_no_change=True,
        )
        assert default_component._get_component_hash(
            keys_to_omit=["creation_context"]
        ) != new_component._get_component_hash(keys_to_omit=["creation_context"])
        assert new_component.description == "description_{0}".format(new_version)
        assert new_component.display_name == "display_name_{0}".format(new_version)
        assert new_component.tags == {"tags": "tags_{0}".format(new_version)}
        assert new_component.version == new_version

        # test component has no change and not use skip_if_no_change parameter
        new_version = randstr("component_version")
        new_component = create_component(client, component_name=component_name, version=new_version)
        assert default_component._get_component_hash(
            keys_to_omit=["creation_context"]
        ) != new_component._get_component_hash(keys_to_omit=["creation_context"])
        assert new_component.version == new_version

    # TODO: code hash will normalize code name to 000000000000000000000, which
    #  is not compatible with mock_component_hash
    @pytest.mark.disable_mock_code_hash
    @pytest.mark.parametrize(
        "component_path",
        [
            pytest.param("./tests/test_configs/components/helloworld_component.yml", id="command"),
            pytest.param(
                "./tests/test_configs/components/helloworld_components_with_env/helloworld_component_env_inline.yml",
                id="command_with_env_inline",
            ),
            pytest.param(
                "./tests/test_configs/components/helloworld_components_with_env/helloworld_component_env_path_1.yml",
                id="command_with_env_path",
            ),
            pytest.param("./tests/test_configs/dsl_pipeline/basic_component/component.yml", id="command_with_code"),
            pytest.param("./tests/test_configs/components/helloworld_parallel.yml", id="parallel_with_code"),
            pytest.param(
                "./tests/test_configs/dsl_pipeline/parallel_component_with_tabular_input/tabular_input_e2e.yml",
                id="parallel_with_env_and_code",
            ),
        ],
    )
    def test_component_download(self, client: MLClient, randstr, component_path: str, request):
        save_dir = Path(f"./tests/test_configs/components/downloaded", request.node.callspec.id)
        temp_component_name = randstr("component_name")

        with tempfile.TemporaryDirectory() as temp_dir:
            download_path = Path(temp_dir)
            component = load_component(component_path)

            component.name = temp_component_name
            created_component = client.components.create_or_update(
                component,
            )
            client.components.download(
                component.name,
                version=component.version,
                download_path=download_path,
            )

            spec_path = download_path.joinpath("component_spec.yaml")
            downloaded_component = load_component(spec_path)
            downloaded_component.name += "_recreated"
            recreated_component = client.components.create_or_update(downloaded_component)

            # normalize component name
            spec_path.write_text(spec_path.read_text().replace(temp_component_name, "random_component_name"))
            if save_dir.is_dir():
                try:
                    for file in save_dir.glob("**/*"):
                        if file.is_file():
                            assert file.read_text() == download_path.joinpath(file.relative_to(save_dir)).read_text(), (
                                f"downloaded component is not the same as last snapshot. "
                                f"Please check if changes under {download_path} are expected."
                            )
                except AssertionError:
                    # replace saved snapshot with downloaded one if not matched
                    shutil.rmtree(save_dir, ignore_errors=True)
                    shutil.copytree(download_path, save_dir)
            else:
                shutil.copytree(download_path, save_dir)

            # compare created component and recreated component after we updated the saved snapshot (if applicable)
            omit_fields = [
                "id",
                "name",
                "creation_context",
            ]
            assert omit_with_wildcard(recreated_component._to_dict(), *omit_fields) == omit_with_wildcard(
                created_component._to_dict(), *omit_fields
            )

    @pytest.mark.parametrize(
        "params_override,expected_attrs",
        [
            pytest.param(
                {
                    "version": "2",
                    "description": "test load component from flow",
                },
                {
                    "version": "2",
                    "description": "test load component from flow",
                },
                id="basic",
            ),
            pytest.param(
                {
                    "azureml": {
                        "environment": "azureml:AzureML-sklearn-1.0-ubuntu20.04-py38-cpu:33",
                        "is_deterministic": False,
                    }
                },
                {
                    "environment": "r:.+/environments/AzureML-sklearn-1.0-ubuntu20.04-py38-cpu/versions/33",
                    "is_deterministic": False,
                },
                id="with_environment",
            ),
        ],
    )
    def test_load_component_from_flow(self, client: MLClient, randstr, params_override: dict, expected_attrs: dict):
        target_path: str = "./tests/test_configs/flows/basic/flow.dag.yaml"
        component = load_component(
            target_path,
            params_override=[
                {
                    "name": randstr("component_name"),
                },
                params_override,
            ],
        )

        created_component = client.components.create_or_update(component)

        for attr, expected_value in expected_attrs.items():
            if isinstance(expected_value, str) and expected_value.startswith("r:"):
                expected_regex = re.compile(expected_value[2:])
                assert expected_regex.match(getattr(created_component, attr)), f"attribute {attr} is not as expected"
            else:
                assert getattr(created_component, attr) == expected_value, f"attribute {attr} is not as expected"

        assert component._get_origin_code_value() == created_component._get_origin_code_value()

    @pytest.mark.live_test_only("Needs re-recording to work with new test proxy sanitizers")
    def test_load_component_from_flow_in_registry(self, registry_client: MLClient, randstr):
        target_path: str = "./tests/test_configs/flows/runs/with_environment.yml"
        component = load_component(
            target_path,
            params_override=[
                {
                    "name": randstr("component_name"),
                    "version": "1",
                    "description": "test load component from flow",
                }
            ],
        )

        created_component = registry_client.components.create_or_update(component, version="2")

        assert created_component.name == component.name
        assert created_component.version == "2"

        assert component._get_origin_code_value() == created_component._get_origin_code_value()
        assert component.environment == created_component.task.environment



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import pytest
import yaml

from azure.ai.ml.entities._feature_store.feature_store import FeatureStore
from azure.ai.ml.entities._load_functions import load_feature_store


@pytest.mark.unittest
@pytest.mark.data_experiences_test
class TestFeatureStoreSchema:
    def test_feature_store_constructor(self):
        from azure.ai.ml.entities import ManagedIdentityConfiguration
        from azure.ai.ml.entities._feature_store._constants import (
            OFFLINE_MATERIALIZATION_STORE_TYPE,
            ONLINE_MATERIALIZATION_STORE_TYPE,
        )
        from azure.ai.ml.entities._feature_store.materialization_store import MaterializationStore
        from azure.ai.ml.exceptions import ValidationException

        IDENTITY_RESOURCE_ID = "/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/test_rg/providers/Microsoft.ManagedIdentity/userAssignedIdentities/exist_identity"
        OFFLINE_STORE_RESOURCE_ID = "/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourceGroups/test_rg/providers/Microsoft.Storage/storageAccounts/test_storage/blobServices/default/containers/exist"
        ONLINE_STORE_RESOURCE_ID = "/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourceGroups/mdctest/providers/Microsoft.Cache/Redis/exist"

        feature_store = FeatureStore(
            name="dummy_name",
            materialization_identity=ManagedIdentityConfiguration(resource_id=IDENTITY_RESOURCE_ID),
            offline_store=MaterializationStore(
                type=OFFLINE_MATERIALIZATION_STORE_TYPE, target=OFFLINE_STORE_RESOURCE_ID
            ),
            online_store=MaterializationStore(type=ONLINE_MATERIALIZATION_STORE_TYPE, target=ONLINE_STORE_RESOURCE_ID),
            description="dummy_description",
            display_name="dummy_display_name",
            identity=ManagedIdentityConfiguration(resource_id=IDENTITY_RESOURCE_ID),
            resource_group="rg",
            location="eastus",
            tags={
                "key": "value",
            },
        )

        assert feature_store.name == "dummy_name"
        assert feature_store.materialization_identity is not None
        assert feature_store.offline_store is not None
        assert feature_store.online_store is not None
        assert feature_store.description == "dummy_description"
        assert feature_store.display_name == "dummy_display_name"
        assert feature_store.identity is not None
        assert feature_store._feature_store_settings is not None
        assert feature_store._feature_store_settings.online_store_connection_name is None
        assert feature_store._feature_store_settings.offline_store_connection_name is None
        assert feature_store._feature_store_settings.compute_runtime is not None
        assert feature_store.location == "eastus"
        assert feature_store.resource_group == "rg"
        assert feature_store.tags is not None
        assert feature_store.hbi_workspace is False
        assert feature_store.storage_account is None
        assert feature_store.container_registry is None
        assert feature_store.key_vault is None
        assert feature_store.application_insights is None
        assert feature_store.customer_managed_key is None
        assert feature_store.image_build_compute is None
        assert feature_store.public_network_access is None
        assert feature_store.primary_user_assigned_identity is None
        assert feature_store.managed_network is None

    def test_feature_store_load(self) -> None:
        test_path = "./tests/test_configs/feature_store/feature_store_full.yaml"
        with open(test_path, "r") as f:
            target = yaml.safe_load(f)
        with open(test_path, "r") as f:
            feature_store: FeatureStore = load_feature_store(source=test_path)
        assert feature_store.name == target["name"]
        assert feature_store.description == target["description"]
        assert feature_store.materialization_identity is not None
        assert feature_store.offline_store is not None
        assert feature_store.tags is not None
        assert feature_store.properties is not None
        assert feature_store.managed_network is not None
        assert feature_store.managed_network.outbound_rules is not None

    def test_materialization_store(self):
        from azure.ai.ml.entities._feature_store._constants import (
            OFFLINE_MATERIALIZATION_STORE_TYPE,
            ONLINE_MATERIALIZATION_STORE_TYPE,
        )
        from azure.ai.ml.entities._feature_store.materialization_store import MaterializationStore
        from azure.ai.ml.exceptions import ValidationException

        with pytest.raises(ValidationException) as ve:
            FeatureStore(
                name="name",
                description="description",
                offline_store=MaterializationStore(
                    type=OFFLINE_MATERIALIZATION_STORE_TYPE, target="offline_store_resource_id"
                ),
            )
        assert "Invalid ARM Id" in str(ve)

        with pytest.raises(ValidationException) as ve:
            FeatureStore(
                name="name",
                description="description",
                online_store=MaterializationStore(
                    type=ONLINE_MATERIALIZATION_STORE_TYPE, target="online_store_resource_id"
                ),
            )
        assert "Invalid ARM Id" in str(ve)



from typing import Optional

import pytest
import json
from marshmallow.exceptions import ValidationError

from azure.ai.ml import load_workspace
from azure.ai.ml._restclient.v2024_10_01_preview.models import (
    Workspace as RestWorkspace,
)
from azure.ai.ml.constants._workspace import FirewallSku, IsolationMode
from azure.ai.ml.entities import ServerlessComputeSettings, Workspace


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestWorkspaceEntity:
    @pytest.mark.parametrize(
        "settings",
        [
            None,
            ServerlessComputeSettings(custom_subnet=None, no_public_ip=False),
            ServerlessComputeSettings(
                custom_subnet="/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/static_resources_cli_v2_e2e_tests_resources/providers/Microsoft.Network/virtualNetworks/testwsvnet/subnets/testsubnet",
                no_public_ip=False,
            ),
            ServerlessComputeSettings(
                custom_subnet="/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/static_resources_cli_v2_e2e_tests_resources/providers/Microsoft.Network/virtualNetworks/testwsvnet/subnets/testsubnetnpip",
                no_public_ip=True,
            ),
        ],
    )
    def test_serverless_compute_settings_loaded_from_rest_object(
        self, settings: Optional[ServerlessComputeSettings]
    ) -> None:
        workspace = Workspace(name="test_workspace", location="test_location", serverless_compute=settings)
        rest_object = workspace._to_rest_object()
        assert rest_object is not None
        if settings is None:
            assert rest_object.serverless_compute_settings is None
        else:
            assert ServerlessComputeSettings._from_rest_object(rest_object.serverless_compute_settings) == settings

    def test_from_rest_object(self) -> None:
        with open("./tests/test_configs/workspace/workspace_full_rest_response.json", "r") as f:
            rest_object = RestWorkspace.deserialize(json.load(f))

        workspace = Workspace._from_rest_object(rest_object)

        assert (
            workspace.id
            == "/subscriptions/sub-id/test_workspace/providers/Microsoft.Storage/storageAccounts/storage-account-name"
        )
        assert workspace.name == "test_workspace"
        assert workspace.location == "test_location"
        assert workspace.description == "test_description"
        assert workspace.tags == {"test_tag": "test_value"}
        assert workspace.display_name == "test_friendly_name"
        assert workspace.discovery_url == "test_discovery_url"
        assert workspace.resource_group == "providers"
        assert workspace.storage_account == "test_storage_account"
        assert workspace.key_vault == "test_key_vault"
        assert workspace.application_insights == "test_application_insights"
        assert workspace.container_registry == "test_container_registry"
        assert workspace.customer_managed_key.key_uri == "key_identifier"
        assert workspace.customer_managed_key.key_vault == "key_vault_arm_id"
        assert workspace.hbi_workspace is True
        assert workspace.public_network_access == "Enabled"
        assert workspace.image_build_compute == "test_image_build_compute"
        assert workspace.discovery_url == "test_discovery_url"
        assert workspace.mlflow_tracking_uri == "ml_flow_tracking_uri"
        assert workspace.primary_user_assigned_identity == "test_primary_user_assigned_identity"
        assert workspace.system_datastores_auth_mode == "AccessKey"
        assert workspace.enable_data_isolation == True
        assert workspace.allow_roleassignment_on_rg == True
        assert workspace._hub_id == "hub_resource_id"
        assert workspace._kind == "project"
        assert workspace._workspace_id == "workspace_id"
        assert workspace.identity is not None
        assert workspace.managed_network is not None
        assert workspace._feature_store_settings is not None
        assert workspace.network_acls is not None
        assert workspace.provision_network_now == True
        assert workspace.serverless_compute is not None
        assert workspace.network_acls is not None

    def test_from_rest_object_for_attributes_none(self) -> None:
        with open("./tests/test_configs/workspace/workspace_full_rest_response.json", "r") as f:
            rest_json = json.load(f)
            del rest_json["properties"]["managedNetwork"]
            del rest_json["properties"]["encryption"]
            rest_json["id"] = "/subscriptions/sub-id"
            del rest_json["identity"]
            del rest_json["properties"]["featureStoreSettings"]
            del rest_json["properties"]["serverlessComputeSettings"]
            del rest_json["properties"]["networkAcls"]
            rest_object = RestWorkspace.deserialize(rest_json)

        workspace = Workspace._from_rest_object(rest_object)

        assert workspace.id == "/subscriptions/sub-id"
        assert workspace.name == "test_workspace"
        assert workspace.location == "test_location"
        assert workspace.description == "test_description"
        assert workspace.tags == {"test_tag": "test_value"}
        assert workspace.display_name == "test_friendly_name"
        assert workspace.discovery_url == "test_discovery_url"
        assert workspace.resource_group is None
        assert workspace.storage_account == "test_storage_account"
        assert workspace.key_vault == "test_key_vault"
        assert workspace.application_insights == "test_application_insights"
        assert workspace.container_registry == "test_container_registry"
        assert workspace.customer_managed_key is None
        assert workspace.hbi_workspace is True
        assert workspace.public_network_access == "Enabled"
        assert workspace.image_build_compute == "test_image_build_compute"
        assert workspace.discovery_url == "test_discovery_url"
        assert workspace.mlflow_tracking_uri == "ml_flow_tracking_uri"
        assert workspace.primary_user_assigned_identity == "test_primary_user_assigned_identity"
        assert workspace.system_datastores_auth_mode == "AccessKey"
        assert workspace.enable_data_isolation == True
        assert workspace.allow_roleassignment_on_rg == True
        assert workspace._hub_id == "hub_resource_id"
        assert workspace._kind == "project"
        assert workspace._workspace_id == "workspace_id"
        assert workspace.identity is None
        assert workspace.managed_network is None
        assert workspace._feature_store_settings is None
        assert workspace.network_acls is None
        assert workspace.provision_network_now == True
        assert workspace.serverless_compute is None

    def test_serverless_compute_settings_subnet_name_must_be_an_arm_id(self) -> None:
        with pytest.raises(ValidationError):
            ServerlessComputeSettings(custom_subnet="justaname", no_public_ip=True)

    @pytest.mark.parametrize(
        "settings",
        [
            ServerlessComputeSettings(
                custom_subnet="/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/static_resources_cli_v2_e2e_tests_resources/providers/Microsoft.Network/virtualNetworks/testwsvnet/subnets/default",
                no_public_ip=True,
            ),  # Override but using same value
            ServerlessComputeSettings(
                custom_subnet="/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/static_resources_cli_v2_e2e_tests_resources/providers/Microsoft.Network/virtualNetworks/testwsvnet/subnets/testsubnet",
                no_public_ip=True,
            ),
            ServerlessComputeSettings(
                custom_subnet="/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/static_resources_cli_v2_e2e_tests_resources/providers/Microsoft.Network/virtualNetworks/testwsvnet/subnets/default",
                no_public_ip=False,
            ),
            ServerlessComputeSettings(
                custom_subnet="/subscriptions/b17253fa-f327-42d6-9686-f3e553e24763/resourcegroups/static_resources_cli_v2_e2e_tests_resources/providers/Microsoft.Network/virtualNetworks/testwsvnet/subnets/testsubnet",
                no_public_ip=False,
            ),
        ],
    )
    def test_workspace_load_override_serverless(self, settings: ServerlessComputeSettings) -> None:
        params_override = [
            {
                "serverless_compute": {
                    "custom_subnet": settings.custom_subnet,
                    "no_public_ip": settings.no_public_ip,
                }
            }
        ]

        workspace_override = load_workspace(
            "./tests/test_configs/workspace/workspace_serverless.yaml",
            params_override=params_override,
        )
        assert workspace_override.serverless_compute == settings

    def test_workspace_load_yamls_to_test_outbound_rule_load(self):
        workspace = load_workspace("./tests/test_configs/workspace/workspace_many_ob_rules.yaml")

        assert workspace.managed_network is not None
        assert workspace.managed_network.isolation_mode == IsolationMode.ALLOW_ONLY_APPROVED_OUTBOUND

        rules = workspace.managed_network.outbound_rules
        assert rules[0].name == "microsoft"
        assert rules[0].destination == "microsoft.com"

        assert rules[1].name == "appGwRule"
        assert rules[1].service_resource_id == "/someappgwid"
        assert rules[1].spark_enabled == False
        assert rules[1].subresource_target == "appGwPrivateFrontendIpIPv4"
        assert "contoso.com" in rules[1].fqdns
        assert "contoso2.com" in rules[1].fqdns

        assert rules[2].name == "servicetag-w-prefixes"
        assert rules[2].service_tag == "sometag"
        assert rules[2].protocol == "TCP"
        assert rules[2].port_ranges == "80, 8080-8089"
        assert "168.63.129.16" in rules[2].address_prefixes
        assert "10.0.0.0/24" in rules[2].address_prefixes

    def test_workspace_load_yamls_to_test_firewallsku_load(self):
        workspace = load_workspace("./tests/test_configs/workspace/workspace_mvnet_with_firewallsku.yaml")

        assert workspace.managed_network is not None
        assert workspace.managed_network.isolation_mode == IsolationMode.ALLOW_ONLY_APPROVED_OUTBOUND
        rules = workspace.managed_network.outbound_rules

        assert workspace.managed_network.firewall_sku == "Basic"

        assert rules[0].name == "microsoft"
        assert rules[0].destination == "microsoft.com"

        assert rules[1].name == "appGwRule"
        assert rules[1].service_resource_id == "/someappgwid"
        assert rules[1].spark_enabled == False
        assert rules[1].subresource_target == "appGwPrivateFrontendIpIPv4"
        assert "contoso.com" in rules[1].fqdns
        assert "contoso2.com" in rules[1].fqdns

        assert rules[2].name == "pytorch"
        assert rules[2].destination == "*.pytorch.org"

    def test_workspace_load_yaml_to_test_network_acls_load(self):
        workspace = load_workspace("./tests/test_configs/workspace/ai_workspaces/workspacehub_with_networkacls.yaml")

        assert workspace.network_acls is not None
        assert len(workspace.network_acls.ip_rules) == 2
        assert workspace.network_acls.default_action == "Deny"
        assert workspace.public_network_access == "Enabled"
        ip_rules = workspace.network_acls.ip_rules

        assert ip_rules[0].value == "103.248.19.87"
        assert ip_rules[1].value == "103.248.19.86/32"
